@@ -42,6 +42,7 @@ typedef struct _bw_map {
 
 SEM bw_lock;
 BW_MAP bw_map;
+int draw_map_flag = 0;
 
 extern SEM res_lock;
 
@@ -236,7 +237,7 @@ int g_len,i;
 			this_coord = 1;
 		}
 		bw1 = *bw;
-		map_conv_reverse(m->map_file,
+		map_conv_reverse(m->map_file,0,
 			bw1.surp.ptr,bw1.surp.reso,SURP_NOS);
 		lock_task(res_lock);
 		m->flags &= ~MF_BROWSE;
@@ -474,7 +475,7 @@ LOAD_RESOURCE_WORK * wp;
 		setup_minrect(m);
 
 		bw1 = *bw;
-		map_conv_reverse(m->map_file,
+		map_conv_reverse(m->map_file,0,
 			bw1.surp.ptr,bw1.surp.reso,SURP_NOS);
 
 		lock_task(res_lock);
@@ -602,7 +603,7 @@ int i;
 			this_coord = 1;
 		}
 		bw1 = *bw;
-		map_conv_reverse(m->map_file,
+		map_conv_reverse(m->map_file,0,
 			bw1.surp.ptr,bw1.surp.reso,SURP_NOS);
 		lock_task(res_lock);
 		m->flags &= ~MF_BROWSE;
@@ -989,11 +990,13 @@ extern int spiral_count;
 GB_RECT * sr;
 int sr_len;
 int f;
+MAP_CONVERT_WORK mcw;
 
 int reso_cnt,minus_cnt,rect_cnt;
 int proc;
 
 
+
 	proc = 0;
 
 	dw->limit_reso = 0;
@@ -1009,6 +1012,7 @@ int proc;
 
 	lock_task(res_lock);
 
+	sr = get_surp_rect(&sr_len,&dw->surp,r);
 	for ( m1 = R_NEXT(MAP*,&r->c.map_children) ; 
 			m1 != (MAP*)&r->c.map_children ;
 			m1 = R_NEXT(MAP*,&m1->h) ) {
@@ -1017,10 +1021,6 @@ int proc;
 			proc = 1;
 			continue;
 		}
-		if ( wf_active_check_2(gf,dw->wfid) < 0 )
-{ss_printf("FREE=RETURN-COORDINATE\n");
-			break;
-}
 		if ( m1->src == 0 ) {
 			if ( m1->src_url.resource && 
 				l_strcmp(&m1->src_url.resource
@@ -1043,8 +1043,6 @@ int proc;
 		if ( (m1->src->h.flags & RF_INDICATE) == RF_I_OFF )
 			continue;
 		setup_minrect(m1);
-
-		sr = get_surp_rect(&sr_len,&dw->surp,r);
 		if  ( check_nan_rect(&m1->map_file->h.minrect) == 0 ) {
 			for ( i = 0 ; i < sr_len ; i ++ ) {
 				if ( cross_rect_rect(
@@ -1061,14 +1059,9 @@ int proc;
 				}
 				else m1->src->draw_gb.hit_outofcircle ++;
 			}
-			if ( sr )
-				d_f_ree(sr);
 			continue;
 		sr_ok:	;
 		}
-		if ( sr )
-			d_f_ree(sr);
-
 		if ( m1->src->h.type == RT_DRAW_GB )
 			m1->src->draw_gb.hit_outofcircle = 0;
 		dw1.limit_reso = -1;
@@ -1085,7 +1078,9 @@ int proc;
 			dw1.map = m1;
 			dw1.limit_reso = 0;
 			dw1.data_request = 0;
-			map_conv_reverse(m1->map_file,
+			mcw.sr = sr;
+			mcw.sr_len = sr_len;
+			map_conv_reverse(m1->map_file,&mcw,
 				dw1.surp.ptr,dw1.surp.reso,SURP_NOS);
 
 			if ( m1->src->h.visible_resolution != 0 &&
@@ -1147,8 +1142,10 @@ rect_cnt ++;
 				continue;
 			}
 
+			mcw.sr = sr;
+			mcw.sr_len = sr_len;
 			map_conv_reverse(
-				m1->map_file,
+				m1->map_file,&mcw,
 				dw1.pt_list,dw1.pt_reso,dw1.size);
 
 			if ( m1->src->h.visible_resolution != 0 &&
@@ -1162,7 +1159,7 @@ rect_cnt ++;
 
 
 			dw1.map = m1;
-			map_conv_reverse(m1->map_file,
+			map_conv_reverse(m1->map_file,&mcw,
 				dw1.surp.ptr,dw1.surp.reso,SURP_NOS);
 			unlock_task(res_lock,"draw_coordinate");
 			draw_resource(
@@ -1178,7 +1175,25 @@ rect_cnt ++;
 		default:
 			er_panic("no support");
 		}
+		if ( draw_map_flag ) {
+			dw1 = *dw;
+			dw1.parent = dw;
+			dw1.pt_list = 0;
+			dw1.pt_reso = 0;
+			dw1.map = m1;
+			dw1.limit_reso = 0;
+			dw1.data_request = 0;
 
+			unlock_task(res_lock,"draw_coordinate");
+
+			draw_resource(
+				gf,
+				m1->map_file,
+				&dw1);
+			lock_task(res_lock);
+			dw->flags |= dw1.flags;
+			dw->data_request += dw1.data_request;
+		}
 		if ( dw1.limit_reso >= 0 ) {
 			map_conv_forward(m1->map_file,
 				&dw1.limit_ptr,
@@ -1189,7 +1204,17 @@ rect_cnt ++;
 				dw->limit_reso = dw1.limit_reso;
 			}
 		}
+
+		unlock_task(res_lock,"draw_coordinate");
+		if ( wf_active_check_2(gf,dw->wfid) < 0 )
+{ss_printf("FREE=RETURN-COORDINATE\n");
+			break;
+}
+		lock_task(res_lock);
+
 	}
+	if ( sr )
+		d_f_ree(sr);
 	if ( proc == 0 && dw->limit_reso == 0 )
 			dw->limit_reso = r->h.visible_resolution*LIMIT_RESOLUTION_RATE_COORDINATE;
 	if ( r->c.flags & CF_FIX_LIMIT_RESO ) {
@@ -1208,6 +1233,7 @@ ss_printf("dw %ls %f %f %f\n",
 get_url_str2(&r->h.entry),
 dw->limit_reso,r->h.limit_resolution,r->h.visible_resolution);
 */
+
 	unlock_task(res_lock,"draw_coordinate");
 
 
@@ -1319,7 +1345,7 @@ GET_POINT_WORK ww;
 			continue;
 		dummy[0] = dummy[1] = 1;
 		memcpy(pp,p,sizeof(p));
-		map_conv_reverse(m1->map_file,pp,dummy,2);
+		map_conv_reverse(m1->map_file,0,pp,dummy,2);
 		if ( dummy[0] < 0 )
 			continue;
 		if ( dummy[1] < 0 )
