@@ -31,6 +31,7 @@
 #include	"pri_level.h"
 #include	"win_flame.h"
 #include	"machine/include.h"
+#include	"mx_format.h"
 
 typedef struct pixel_req {
 	struct pixel_req *	next;
@@ -94,15 +95,18 @@ gv_new_luster_option(RESOURCE * r,int r64_level)
 	case 'P':
 	case 'G':
 	case 'B':
-		r->pr64.mtx = open_matrix(get_url_str2(&r->h.target),0,0,
+		r->h.mtx = open_matrix(get_url_str2(&r->h.target),0,0,
 						open_old_matrix,get_omx_r64_work(r));
 		break;
 	case '0':
 	case '1':
 	case '2':
-		r->pr64.mtx = open_matrix(get_url_str2(&r->h.target),0,0,
+		r->h.mtx = open_matrix(get_url_str2(&r->h.target),0,0,
 						open_old_matrix,get_omx_cr_work(r));
 		break;
+	case 'M':
+		insert_resource_matrix(r,1);
+		break;
 	default:
 		er_panic("gv_new_luster_option(1)");
 	}
@@ -323,6 +327,7 @@ typedef struct mtx_cache {
 	MATRIX_DH_SET	ds;
 	MATRIX_NODE *	n;
 	unsigned long *	ds_ptr;
+	unsigned	leaf;
 } MTX_CACHE;
 
 unsigned long
@@ -339,7 +344,7 @@ INTEGER64 dim_code[3];
 int xx,yy;
 INTEGER64 mx,my;
 int clr;
-	m = r->pr64.mtx;
+	m = r->h.mtx;
 	clr = 0;
 	if ( cache->n ) {
 		if ( cache->level == lev &&
@@ -358,7 +363,7 @@ int clr;
 //ss_printf("get_mtx_data-1\n");
 
 	d = get_matrix_node_channel
-			(&err,&cache->n,r->pr64.mtx,
+			(&err,&cache->n,r->h.mtx,
 			&dim_code[0],CH_VISU,
 			GN_TREE,GN_NODE_CREATE,0);
 	if ( d == 0 ) {
@@ -395,8 +400,96 @@ end:
 	return cache->ds_ptr[xx + yy * cache->ds.ix[0]];
 }
 
+
+unsigned long
+convert_rgba_to_cc(unsigned int cc)
+{
+unsigned int r,g,b;
+	if ( (cc & 0xff) >= 0x80 )
+		return C_TRANSPARENT;
+	r = (cc >> 24)&0x0ff;
+	g = (cc >> 16)&0x0ff;
+	b = (cc >> 8)&0x0ff;
+	return ((r << (COL_BIT-8))) |
+		((g << (COL_BIT-8)) << COL_BIT) |
+		((b << (COL_BIT-8)) << (COL_BIT*2));
+}
+
+
+
+unsigned long
+get_mtx_data_2(MTX_CACHE * cache,RESOURCE * r,INTEGER64 lev,INTEGER64 x,INTEGER64 y,INTEGER64 * lev_p);
+
+unsigned long
+get_mtx_data_2(MTX_CACHE * cache,RESOURCE * r,INTEGER64 lev,INTEGER64 x,INTEGER64 y,INTEGER64 * lev_p)
+{
+int err;
+void * d;
+MATRIX * m;
+MATRIX_NODE * nn;
+INTEGER64 dim_code[3];
+int xx,yy;
+INTEGER64 mx,my;
+int clr;
+	m = r->h.mtx;
+	clr = 0;
+	if ( cache->n ) {
+		if ( cache->level == lev &&
+			cache->tl_x <= x && cache->tl_y <= y &&
+			cache->br_x > x && cache->br_y > y )
+			goto end;
+		clr = 1;
+		matrix_node_channel_unlock(cache->n,0);
+		unlock_node(cache->n,0);
+	}
+	nn = cache->n;
+	
+	dim_code[0] = lev;
+	dim_code[1] = x;
+	dim_code[2] = y;
+//ss_printf("get_mtx_data-1\n");
+
+	d = get_matrix_node_channel
+			(&err,&cache->n,r->h.mtx,
+			&dim_code[0],r->h.visu_channel,
+			GN_TREE,GN_NODE_CREATE,0);
+	if ( d == 0 ) {
+		if ( nn && clr == 0 ) {
+			matrix_node_channel_unlock(nn,0);
+			unlock_node(nn,0);
+		}
+		memset(cache,0,sizeof(*cache));
+		*lev_p = m->total_levels;
+		return C_TRANSPARENT;
+	}
+	get_matrix_dh_set(&cache->ds,d);
+	dim_code[0] = cache->n->dim_code[0];
+	dim_code[1] = cache->n->dim_code[1];
+	dim_code[2] = cache->n->dim_code[2];
+	cache->tl_x = dim_code[1] & (mx=(-(1<<(dim_code[0]*m->dim_divide[0] + m->block_size[0]))));
+	cache->tl_y = dim_code[2] & (my=(-(1<<(dim_code[0]*m->dim_divide[1] + m->block_size[1]))));
+	cache->br_x = cache->tl_x + (1<<(dim_code[0]*m->dim_divide[0] + m->block_size[0]));
+	cache->br_y = cache->tl_y + (1<<(dim_code[0]*m->dim_divide[1] + m->block_size[1]));
+if ( (x & mx) != cache->tl_x || (y & my) != cache->tl_y )
+er_panic("get_mtx_data");
+	if ( cache->br_x > m->pixel_size[0] )
+		cache->br_x = m->pixel_size[0];
+	if ( cache->br_y > m->pixel_size[1] )
+		cache->br_y = m->pixel_size[1];
+	cache->level = dim_code[0];
+	cache->ds_ptr = cache->ds.offset;
+	cache->rect_logsize = m->block_size[0];
+	*lev_p = dim_code[0];
+
+end:
+	xx = (x - cache->tl_x)>>(cache->level*m->dim_divide[0]);
+	yy = (y - cache->tl_y)>>(cache->level*m->dim_divide[1]);
+	return convert_rgba_to_cc(cache->ds_ptr[xx + yy * cache->ds.ix[0]]);
+}
+
+
 void
-draw_pixel_data(
+draw_pixel_data_1(
 	GBVIEW_FLAME * gf,
 	RESOURCE* r,
 	DRAW_WORK * dw)
@@ -536,6 +629,191 @@ int tim,tim2;
 }
 
 
+void
+draw_pixel_data_2(
+	GBVIEW_FLAME * gf,
+	RESOURCE* r,
+	DRAW_WORK * dw)
+{
+int i;
+GB_POINT p;
+REAL1 res;
+INTEGER64 xx,yy;
+REAL1 dpm1,dpm2;
+INTEGER64 level;
+float d1,d2,d;
+unsigned long cc;
+INTEGER64 lev;
+int lev_cnt;
+MTX_CACHE cache;
+int tim,tim2;
+
+MATRIX * mtx;
+float divide;
+int int_divide;
+
+
+	if ( r->pr64.exit_lock )
+		return;
+	if ( get_mov_flag(gf,dw->wfid) != 0 )
+		return;
+	if ( r->h.mtx == 0 )
+		return;
+	mtx = r->h.mtx;
+
+	divide = 1;
+	for ( i = 0 ; i < mtx->dim_divide[0] ; i ++ )
+		divide *= 2;
+	int_divide = mtx->dim_divide[0];
+
+	dw->limit_reso = r->pr64.dpm;
+	dw->limit_ptr = dw->pt_list[dw->size/2];
+	r->pr64.loop_no ++;
+	dw->flags |= WFF_LUSTER; 
+	dpm1 = r->pr64.dpm;
+	dpm2 = dpm1;
+	level = 0;
+	d1 = 1;
+	d2 = 0;
+
+	res = -1;
+	for ( i = 0 ; i < dw->size ; i ++ ) {
+		if ( res < dw->pt_reso[i] )
+			res = dw->pt_reso[i];
+	}
+
+/*
+	if ( res > dpm1*LIMIT_RESOLUTION_RATE )
+		return;
+*/
+
+	if ( dpm2 < res || res < dpm2/divide ) {
+		if ( res > dpm1 ) {
+			dpm2 = dpm1;
+		level = 0;
+		}
+		else {
+			level = 0;
+			d = -1;
+			for ( dpm2 = dpm1 ;
+				res <= dpm2 &&
+				level < r->pr64.max_level;
+				d = dpm2,
+				dpm2 /= divide,
+				level ++ );
+			dpm2 = d;
+			level --;
+		}
+	}
+
+	lock_task(luster_lock);
+//	lock_task(tc_lock);
+	lev_cnt = 0;
+	memset(&cache,0,sizeof(cache));
+	lev = level;
+	tim = get_xltime();
+	for ( i = 0 ; i < dw->size ; i ++ ) {
+		if ( (i % 1000) == 0 && wf_active_check_2(gf,dw->wfid) < 0 )
+{ss_printf("FREE=RETURN-PIXEL\n");
+			break;
+}
+		if ( r->pr64.exit_lock )
+			break;
+		tim2 = get_xltime();
+		if ( tim != tim2 ) {
+			if ( get_mov_flag(gf,dw->wfid) != 0 )
+				break;
+			tim = tim2;
+		}
+		p = dw->pt_list[i];
+		if ( dw->pt_reso[i] < 0 )
+			continue;
+		xx = rint(p.x*r->pr64.dpm);
+		yy = rint(p.y*r->pr64.dpm);
+		if ( xx < 0 || yy < 0 )
+			continue;
+		if ( xx >= mtx->pixel_size[0] )
+			continue;
+		if ( yy >= mtx->pixel_size[1] )
+			continue;
+		if ( lev_cnt && cache.ds_ptr ) {
+		int lx,ly;
+			if (		cache.tl_x <= xx && xx < cache.br_x &&
+					cache.tl_y <= yy && yy < cache.br_y ) {
+				if ( cache.level == lev ) {
+
+					lx = (xx>>(lev*int_divide))&((1<<mtx->block_size[0])-1);
+					ly = (yy>>(lev*int_divide))&((1<<mtx->block_size[1])-1);
+					if ( lx >= cache.ds.ix[0] ) {
+						cc = C_TRANSPARENT;
+					}
+					else if ( ly >= cache.ds.ix[1] ) {
+						cc = C_TRANSPARENT;
+					}
+					else cc = convert_rgba_to_cc(cache.ds_ptr[lx + 
+						ly*cache.ds.ix[0]]);
+				}
+				else if ( cache.leaf ) {
+					lx = (xx>>(cache.level*int_divide))&((1<<mtx->block_size[0])-1);
+					ly = (yy>>(cache.level*int_divide))&((1<<mtx->block_size[1])-1);
+					if ( lx >= cache.ds.ix[0] ) {
+						cc = C_TRANSPARENT;
+					}
+					else if ( ly >= cache.ds.ix[1] ) {
+						cc = C_TRANSPARENT;
+					}
+					else cc = convert_rgba_to_cc(cache.ds_ptr[lx + 
+						ly*cache.ds.ix[0]]);
+				}
+				else	goto non_cache;
+			}
+			else goto non_cache;
+			lev_cnt ++;
+			if ( lev_cnt >= 100 )
+				lev_cnt = 0;
+		}
+		else {
+		non_cache:
+//			cc = _tc_get_pixel(&r->pr64.tc,level,xx,yy,&lev);
+			cc = get_mtx_data_2(&cache,r,level,xx,yy,&lev);
+//ss_printf("CC %lli %lli %lli %x\n",xx,yy,level,cc);
+			if ( level != lev && cache.leaf == 0 )
+				dw->data_request ++;
+//			cache = r->pr64.tc;
+			lev_cnt ++;
+		}
+		dw->pixels[i] = cc;
+		dw->flags |= WFF_DRAW;
+	}
+//	unlock_task(tc_lock,"draw_pixel");
+
+	if ( cache.n ) {
+		matrix_node_channel_unlock(cache.n,0);
+		unlock_node(cache.n,0);
+	}
+	
+
+	unlock_task(luster_lock,"draw_pixel");
+
+}
+
+
+void
+draw_pixel_data(
+	GBVIEW_FLAME * gf,
+	RESOURCE* r,
+	DRAW_WORK * dw)
+{
+	switch ( r->pr64.type ) {
+	case 'M':
+		draw_pixel_data_2(gf,r,dw);
+		break;
+	default:
+		draw_pixel_data_1(gf,r,dw);
+		break;
+	}
+}
+
 #else
 void
 draw_pixel_data(
