@@ -26,10 +26,10 @@
 #include	"lock_level.h"
 #include	"memory_debug.h"
 
-/*
+
 #define GVHA_PATH_DEBUG
 #define GVHA_LOADING_STATUS
-*/
+
 
 #define TOLL	5
 #define ACCESS_TIME_LIMIT	20
@@ -39,7 +39,7 @@
 #define SIZE_TICK_INTERVAL	6
 
 #define ECT_UPDATE_TIMEOUT	15
-
+#define FRAME_LOADING_TIMEOUT	20
 
 typedef struct gvha_status {
 	int			status;
@@ -109,19 +109,20 @@ typedef struct gvha_query {
 #define GVHA_MODE_INFO2		3
 #define GVHA_MODE_RSS		4
 #define GVHA_MODE_RSS_SHORT	5
+#define GVHA_MODE_RSS_TINNY	6
 
-#define GVHA_MODE_ECT_LOW	6
+#define GVHA_MODE_ECT_LOW	7
 
-#define GVHA_MODE_ECT_OPEN	(6|GVHA_NREQ_FRAME_CACHE)
-#define GVHA_MODE_ECT_INSERT	7
-#define GVHA_MODE_ECT_UPDATE	(8|GVHA_NREQ_FRAME_CACHE)
-#define GVHA_MODE_ECT_DELETE	(9|GVHA_NREQ_FRAME_CACHE)
-#define GVHA_MODE_ECT_MOVE	10
-#define GVHA_MODE_ECT_SAVE	(11|GVHA_NREQ_FRAME_CACHE)
-#define GVHA_MODE_ECT_CLOSE	(12|GVHA_NREQ_FRAME_CACHE)
-#define GVHA_MODE_ECT_OBJECT	(13|GVHA_NREQ_FRAME_CACHE)
+#define GVHA_MODE_ECT_OPEN	(8|GVHA_NREQ_FRAME_CACHE)
+#define GVHA_MODE_ECT_INSERT	9
+#define GVHA_MODE_ECT_UPDATE	(10|GVHA_NREQ_FRAME_CACHE)
+#define GVHA_MODE_ECT_DELETE	(11|GVHA_NREQ_FRAME_CACHE)
+#define GVHA_MODE_ECT_MOVE	12
+#define GVHA_MODE_ECT_SAVE	(13|GVHA_NREQ_FRAME_CACHE)
+#define GVHA_MODE_ECT_CLOSE	(14|GVHA_NREQ_FRAME_CACHE)
+#define GVHA_MODE_ECT_OBJECT	(15|GVHA_NREQ_FRAME_CACHE)
 
-#define GVHA_MODE_ECT_HIGH	13
+#define GVHA_MODE_ECT_HIGH	16
 
 	unsigned	complete:1;
 	GVHA_POS	pos;
@@ -143,6 +144,7 @@ typedef struct gvha_query {
 	char *		e_information;
 	int		e_objCode;
 	int		e_vct;
+	char *		e_vct_byname;
 	int		e_markTransparent;
 	char *		e_markImg;
 } GVHA_QUERY;
@@ -180,7 +182,9 @@ typedef struct gvha_frame_cache {
 
 	int				complete_cnt;
 
-	unsigned int			access_time;
+	INTEGER64			access_time;
+	INTEGER64			internal_access_time;
+	int				i_interval;
 
 	unsigned 			mv_lock:1;
 	unsigned			lock:1;
@@ -202,6 +206,7 @@ typedef struct gvha_set {
 
 SEM	gvha_lock;
 GVHA_SET *	gvha_set_list;
+int frame_loading_flag;
 
 XLISP_ENV * HTTP_setup_env;
 
@@ -225,7 +230,7 @@ void call_gv_flame_event();
 GVHA_FRAME_CACHE *
 search_gvha_frame_cache_no_access(
 	GVHA_FRAME_CACHE * root,I_POINT sz);
-void gvha_flush_pos(GVHA_FRAME_CACHE * c);
+void gvha_flush_pos(GVHA_FRAME_CACHE * c,int *);
 char *
 get_next_target(GVHA_SET*,GVHA_FRAME_CACHE * c,L_CHAR * path,int mode);
 void unlock_frame_lock(GVHA_FRAME_CACHE * c);
@@ -234,7 +239,7 @@ void gvha_insert_pos(GVHA_FRAME_CACHE * c);
 
 
 void
-get_gvha_vct(GVHA_SET*,GVHA_STATUS * sts,GVHA_VCT_SET*,int vct);
+get_gvha_vct(GVHA_SET*,GVHA_STATUS * sts,GVHA_VCT_SET*,int vct,char*vct_byname);
 GB_POINT
 get_gvha_mark_point(GVHA_STATUS * sts,GVHA_FRAME_CACHE * c,GVHA_VCT_SET * vs,I_POINT display_ptr);
 int
@@ -245,6 +250,10 @@ void
 gvha_update_information(GVHA_STATUS * sts,GVHA_VCT_SET * vs,int code,char * info);
 void
 gbha_delete_mark(GVHA_STATUS * sts,GVHA_VCT_SET * vs,int code);
+void
+frame_loading_tick();
+void
+loading_status_cal(GVHA_FRAME_CACHE  *c);
 
 
 void
@@ -507,11 +516,12 @@ GVHA_FRAME_SIZE * ret;
 }
 
 void
-gvha_frame_size_tick_set(GVHA_SET * g_set)
+gvha_frame_size_tick_set(GVHA_SET * g_set,int *fp_flag)
 {
 GVHA_FRAME_SIZE * s;
 int cnt;
 GVHA_FRAME_CACHE * c;
+//char * tmp;
 	set_size_list_avg_count(g_set);
 	for ( cnt = 0 ; cnt < SIZE_TICK_MAX ; cnt ++ ) {
 		s = search_gvha_frame_size_max(g_set);
@@ -529,7 +539,7 @@ GVHA_FRAME_CACHE * c;
 		c->sts.flags = SF_WIDTH|SF_HEIGHT;
 		wf_set_status(c->gf,&c->sts);
 		locked_wf_status(c->gf,&c->sts);
-		gvha_flush_pos(c);
+		gvha_flush_pos(c,fp_flag);
 /*
 		tmp = get_next_target(c,_dir,GVHA_MODE_RSS);
 		if ( tmp )
@@ -546,7 +556,7 @@ gvha_frame_size_tick()
 {
 GVHA_SET * g_set;
 	for ( g_set = gvha_set_list ; g_set ; ) {
-		gvha_frame_size_tick_set(g_set);
+		gvha_frame_size_tick_set(g_set,0);
 		lock_task(gvha_lock);
 		g_set = g_set->next;
 		unlock_task(gvha_lock,"gvha_frame_size_tick_set");
@@ -666,6 +676,7 @@ gvha_pos_cmp(GVHA_POS * p1,GVHA_POS * p2,int radius)
 double dd;
 double ret;
 
+
 	if ( l_strcmp(p1->target,p2->target) )
 		return -1;
 	dd = p1->resolution - p2->resolution;
@@ -675,7 +686,7 @@ double ret;
 	dd = p1->rotate - p2->rotate;
 	if ( dd < 0 )
 		dd = - dd;
-	ret += (dd * 3.14 * radius)/180;
+	ret += (dd * radius);
 	dd = (p1->center.x - p2->center.x) * p1->resolution;
 	if ( dd < 0 )
 		dd = - dd;
@@ -722,6 +733,8 @@ free_gvha_query(GVHA_QUERY * q)
 	}
 	if ( q->e_markImg )
 		d_f_ree(q->e_markImg);
+	if ( q->e_vct_byname )
+		d_f_ree(q->e_vct_byname);
 }
 
 GVHA_FRAME_CACHE *
@@ -736,7 +749,9 @@ double dist,d;
 double dist_crd,dist_crd_2;
 int ws_flag;
 int tim;
+int lf;
 
+lf = layer_flag;
 	ws_flag = layer_flag & 0x4;
 	layer_flag &= 3;
 
@@ -771,7 +786,6 @@ retry:
 			for ( p = (*rp)->pos ; p ; p = p->next ) {
 				d = gvha_pos_cmp(p,&q->pos,
 					q->win_size.x + q->win_size.y);
-ss_printf("d-A = %f %i\n",d,tim);
 				if ( d < 0 ) {
 					if ( dist_crd == -1 ||
 							tim < dist_crd ) {
@@ -841,7 +855,7 @@ ss_printf("d-A = %f %i\n",d,tim);
 	*rp = ret->next;
 	ret->next = *root;
 	*root = ret;
-ss_printf("RET %p %p\n",ret,ret->gf);
+ss_printf("RET %i - %i\n",ret->gf->id,lf);
 	for ( ; ret->lock ; ) {
 		if ( ret->mv_lock )
 			goto retry;
@@ -956,6 +970,26 @@ complete_unlock(GVHA_FRAME_CACHE * c)
 	unlock_task(gvha_lock,"c");
 }
 
+char *
+get_str_len(char ** ptr,char * ch,char * inp,char * delimit)
+{
+char * ret;
+int len;
+int dlen;
+	dlen = strlen(delimit);
+	for ( len = 0 ; inp[len] ; len ++ ) {
+		if ( memcmp(&inp[len],delimit,dlen) == 0 )
+			break;
+	}
+	*ch = inp[len];
+	ret = d_alloc(len+1);
+	memcpy(ret,inp,len);
+	ret[len] = 0;
+	if ( inp[len] == 0 )
+		*ptr = &inp[len];
+	else	*ptr = &inp[len+1];
+	return ret;
+}
 
 char *
 get_str(char ** ptr,char * ch,char * inp,char * list)
@@ -1147,6 +1181,8 @@ int sz;
 			dir ++;
 			str = get_str(&dir,&ch,dir,"&");
 			q->e_vct = atoi(str);
+			if ( q->e_vct == 0 )
+				q->e_vct_byname = convert_percent(str,0);
 			d_f_ree(str);
 		}
 		else if ( strcmp_ws(&sz,dir,"markImg") == 0 ) {
@@ -1156,6 +1192,7 @@ int sz;
 			dir ++;
 			str = get_str(&dir,&ch,dir,"&");
 			q->e_markImg = convert_percent(str,0);
+			d_f_ree(str);
 		}
 		else if ( strcmp_ws(&sz,dir,"markTransparent") == 0 ) {
 			dir += sz;
@@ -1172,7 +1209,7 @@ int sz;
 			if ( *dir != '=' )
 				return -2;
 			dir ++;
-			str = get_str(&dir,&ch,dir,"_");
+			str = get_str_len(&dir,&ch,dir,"_a_");
 			if ( '0' <= str[0] && str[0] <= '9' ) {
 				q->pos.target = get_crd_by_id(g_set,atoi(str));
 				if ( q->pos.target == 0 )
@@ -1247,7 +1284,8 @@ int sz;
 					q->mode = GVHA_MODE_RSS;
 				else if ( strcmp(str,"rss_short") == 0 )
 					q->mode = GVHA_MODE_RSS_SHORT;
-
+				else if ( strcmp(str,"rss_tinny") == 0 )
+					q->mode = GVHA_MODE_RSS_TINNY;
 				else if ( strcmp(str,"open") == 0 )
 					q->mode = GVHA_MODE_ECT_OPEN;
 				else if ( strcmp(str,"insert") == 0 )
@@ -1417,6 +1455,13 @@ int i;
 	g_set->processed_cache_size ++;
 	_search_gvha_frame_size(g_set,ret->win_size,SF_COUNT|SF_REAL_INS);
 	unlock_task(gvha_lock,"new_frame_cache");
+/*
+ss_printf("INSERT FRAME LOADING\n");
+	if ( frame_loading_flag == 0 ){
+		new_tick(frame_loading_tick,0,0);
+		frame_loading_flag = 1;
+	}
+*/
 	return ret;
 }
 
@@ -1527,6 +1572,135 @@ id_error:
 		n_get_string("duplicate id"));
 }
 
+
+void
+set_wakeup_time(INTEGER64 * wt,INTEGER64 target)
+{
+	if ( *wt == 0 ) {
+		*wt = target;
+	}
+	else if ( *wt > target ) {
+		*wt = target;
+	}
+}
+
+void
+frame_loading_task()
+{
+XL_INTERPRETER * xli;
+GVHA_FRAME_CACHE * c;
+GVHA_SET * gs;
+INTEGER64 n;
+INTEGER64 wt;
+GB_RECT rr;
+GBVIEW_PLANE gbp;
+INTEGER64 loop_time;
+
+	xli = new_xl_interpreter();
+	xli->a_type = XLA_SELF;
+	setup_i(xli);
+
+	loop_time = get_xltime();
+	wt = 0;
+	for ( ; get_xltime() - loop_time < 2 ; ) {
+		lock_task(gvha_lock);
+		n = get_xltime();
+		for ( gs = gvha_set_list ; gs ; gs = gs->next ) {
+			c = gs->processed_cache;
+			for ( ; c ; c = c->next ) {
+				if ( n - c->access_time < FRAME_LOADING_TIMEOUT ) {
+					set_wakeup_time(&wt,c->access_time 
+						+ FRAME_LOADING_TIMEOUT);
+					continue;
+				}
+				if ( c->internal_access_time + c->i_interval > n ) {
+					set_wakeup_time(&wt,
+						c->internal_access_time + c->i_interval);
+					continue;
+				}
+				if ( c->lock ) {
+					set_wakeup_time(&wt,
+						n + FRAME_LOADING_TIMEOUT);
+					continue;
+				}
+				goto internal;
+			}
+		}
+		unlock_task(gvha_lock,"frame_loading_task");
+		break;
+	internal:
+		c->lock = 1;
+		unlock_task(gvha_lock,"frame_loading_task");
+
+ss_printf("FRAME LOADING %i\n",c->gf->id);
+		rr.tl.x = rr.tl.y = 0;
+		rr.br.x = c->win_size.x;
+		rr.br.y = c->win_size.y;
+
+		get_rc_loading_status(
+			(RADAR_CACHE*)c->gf->radar_ptr,&c->ls.rc);
+		wf_free_status(&c->sts);
+		locked_wf_status(c->gf,&c->sts);
+		c->ls.wf = c->sts.loading_status;
+		loading_status_cal(c);
+
+
+		set_redraw_rect(c->gf,&rr);
+
+		if ( ! wf_redraw(c->gf,&gbp,
+				0,0,c->win_size.x,c->win_size.y) )
+			er_panic("GVHA_MODE_MOVE");
+
+		wf_free_plane(&gbp);
+
+wf_wakeup(c->gf);
+
+		
+		lock_task(gvha_lock);
+
+		c->internal_access_time = get_xltime();
+		if ( c->ls.wf.dirty ) {
+ss_printf("DIRTY\n");
+			c->i_interval --;
+			if ( c->i_interval <= 0 )
+				c->i_interval = 1;
+			set_wakeup_time(&wt,
+				c->internal_access_time + c->i_interval);
+		}
+		else {
+			if ( c->i_interval <= 0 )
+				c->i_interval = 1;
+			c->i_interval *= 2;
+			if ( c->i_interval >= FRAME_LOADING_TIMEOUT )
+				c->i_interval = FRAME_LOADING_TIMEOUT;
+			set_wakeup_time(&wt,
+				c->internal_access_time + c->i_interval);
+		}
+		unlock_task(gvha_lock,"frame_loading_task");
+		unlock_frame_lock(c);
+	}
+	close_self_interpreter();
+
+ss_printf("TICK=2 %lli ===================================================\n",wt);
+	if ( wt ){
+		n = get_xltime();
+ss_printf("TICK %i\n",(int)(wt-n));
+		if ( wt <= n )
+			new_tick(frame_loading_tick,-1,0);
+		else	new_tick(frame_loading_tick,(int)-(wt-n),0);
+	}
+}
+
+
+
+void
+frame_loading_tick()
+{
+ss_printf("FRAME LOADING TICK--\n");
+	create_task(frame_loading_task,0,PRI_FETCH);
+}
+
+
 typedef struct _Cache_WORK {
 	int		_g;
 	XLISP_ENV *	e;
@@ -1538,6 +1712,7 @@ typedef struct _Cache_WORK {
 	GVHA_SET *	g_set;
 } Cache_WORK;
 
+
 void
 Cache_thread(TKEY d)
 {
@@ -1545,7 +1720,6 @@ Cache_WORK * w;
 GVHA_FRAME_CACHE *c;
 XL_SEXP * ret, * sym;
 XL_INTERPRETER * xli;
-
 	xli = new_xl_interpreter();
 	xli->a_type = XLA_SELF;
 	setup_i(xli);
@@ -1897,8 +2071,9 @@ L_CHAR * site;
 	if ( ls == 0 )
 		er_panic("get_next_target");
 	id = get_id_by_crd(g_set,ls->entry_url);
+ss_printf("ENTRY URL %ls == %i\n",ls->entry_url,id);
 	if ( id < 0 ) {
-		str = n_string(std_cm,ls->entry_url);
+		str = convert_percent_reverse(n_string(std_cm,ls->entry_url));
 		base = d_re_alloc(base,p+(len=strlen(str))+100);
 		sprintf(&base[p],"s=%i_%i&t=%s_a_%f_%f_%f_%f&l=",
 			c->win_size.x,
@@ -1908,6 +2083,7 @@ L_CHAR * site;
 			c->sts.flame_base_center.y,
 			c->sts.flame_base_rotate,
 			c->sts.flame_base_resolution);
+		d_f_ree(str);
 	}
 	else {
 		base = d_re_alloc(base,p+100);
@@ -1952,11 +2128,11 @@ L_CHAR * site;
 	for ( i = 0 ; i < c->layer_len ; i ++ ) {
 		if ( (c->layer[i].status & IDSS_VISIBLE) == 0 )
 			continue;
-		base = d_re_alloc(base,p+10);
+		base = d_re_alloc(base,p+30);
 		sprintf(&base[p],"%i%%5B1%%5D",c->layer[i].org_id);
 		p = strlen(base);
 	}
-	base = d_re_alloc(base,p+20);
+	base = d_re_alloc(base,p+50);
 	switch ( mode ) {
 	case GVHA_MODE_MOVE:
 		break;
@@ -1972,6 +2148,9 @@ L_CHAR * site;
 	case GVHA_MODE_RSS_SHORT:
 		sprintf(&base[p],"&md=rss_short");
 		break;
+	case GVHA_MODE_RSS_TINNY:
+		sprintf(&base[p],"&md=rss_tinny");
+		break;
 	default:
 		er_panic("get_next_taerget");
 	}
@@ -1989,6 +2168,8 @@ gvha_format_HTML(XL_SEXP * inp)
 {
 STREAM * st;
 XL_SEXP * ret;
+
+
 	st = s_open_string_write(&int_cm);
 	if ( get_type(inp) == XLT_PAIR ) {
 		if ( get_type(car(inp)) == XLT_SYMBOL ) {
@@ -2017,6 +2198,10 @@ char code[10];
 RESOURCE * r;
 OBJ * obj;
 int res_crd_id;
+RESOURCE * locked_r;
+
+	if ( mode == GVHA_MODE_RSS_TINNY )
+		return 0;
 	el = wf_all_plot_element(c->gf);
 	if ( el == 0 )
 		return 0;
@@ -2024,6 +2209,7 @@ int res_crd_id;
 
 	area_i = 1;
 	ret = 0;
+	locked_r = 0;
 	for ( el_1 = el ; el_1 ; el_1 = el_1->next ) {
 		area = n_get_symbol("area");
 		sprintf(name,"area%i",area_i++);
@@ -2068,24 +2254,35 @@ int res_crd_id;
 		if ( mode == GVHA_MODE_RSS_SHORT )
 			goto simple;
 
-		lock_pdb_lock(r,0);
+		if ( r != locked_r ) {
+			if ( locked_r )
+				unlock_pdb_lock(locked_r);
+			lock_pdb_lock(r,0);
+			locked_r = r;
+		}
 		obj = search_obj(r,el_1->code&MAIN_CODE);
 		if ( obj == 0 ) {
-			unlock_pdb_lock(r);
+//			unlock_pdb_lock(r);
 			goto simple;
 		}
 		if ( obj->h.info_org == 0 ) {
-			unlock_pdb_lock(r);
+//			unlock_pdb_lock(r);
 			goto simple;
 		}
 		if ( obj->h.info_card == 0 )
 			ret = cons(List(area,obj->h.info_org,-1),ret);
-		else	ret = cons(List(area,obj->h.info_org,gvha_format_HTML(format_HTML(obj->h.info_card)),-1),ret);
-		unlock_pdb_lock(r);
+		else	ret = cons(List(area,obj->h.info_org,
+				gvha_format_HTML(format_HTML(obj->h.info_card)),-1)
+				,ret);
+//		unlock_pdb_lock(r);
 		continue;
 	simple:
 		ret = cons(List(area,-1),ret);
 	}
+
+	if ( locked_r )
+		unlock_pdb_lock(locked_r);
+
 	ret = List(cons(n_get_symbol("gb:html-map"),ret),-1);
 
 	gc_pop(ret,gc_gb_sexp);
@@ -2094,12 +2291,15 @@ int res_crd_id;
 
 
 XL_SEXP *
-get_layers_info(GBVIEW_STATUS * sts)
+get_layers_info(GBVIEW_STATUS * sts,int mode)
 {
 XL_SEXP * ret;
 GBVIEW_LAYER_STATUS * ls;
 XL_SEXP * sym;
 char buf[10];
+
+	if ( mode == GVHA_MODE_RSS_TINNY )
+		return 0;
 	ret = 0;
 	for ( ls = sts->layers ; ls ; ls = ls->next ) {
 		sym = n_get_symbol("layer");
@@ -2111,19 +2311,19 @@ char buf[10];
 		set_attribute(sym,l_string(std_cm,"flags"),l_string(std_cm,buf));
 		ret = cons(List(sym,-1),ret);
 	}
-ss_printf("layers = ");
-print_sexp(s_stdout,ret,0);
-ss_printf("\n");
 	return reverse(ret);
 }
 
 XL_SEXP *
-return_rdf(GVHA_SET * g_set,GVHA_QUERY * q,GVHA_FRAME_CACHE * c,L_CHAR *path)
+return_rdf(GVHA_SET * g_set,GVHA_QUERY * q,GVHA_FRAME_CACHE * c,L_CHAR *path,int * fp_flag)
 {
 XL_SEXP * ret,* sym,* sym2;
 XL_SEXP * cur, * channel, * item, * item2;
 char * next_target;
 char * rss_target;
+
+	if ( fp_flag && *fp_flag )
+		gvha_flush_pos(c,fp_flag);
 	next_target = get_next_target(g_set,c,path,GVHA_MODE_MOVE);
 	rss_target = get_next_target(g_set,c,path,GVHA_MODE_RSS);
 	if ( next_target == 0 || rss_target == 0 ) {
@@ -2194,7 +2394,9 @@ char * rss_target;
 			get_integer(q->n,0),
 			-1),
 		cons(n_get_symbol("gb:status-layers"),
-			get_layers_info(&c->sts)),
+			get_layers_info(&c->sts,q->mode)),
+		cons(n_get_symbol("gb:status-projection-type"),
+			get_integer(c->sts.flame_base_display_map_type,0)),
 		-1);
 	item = append(item,
 		plot_element_area_list(g_set,c,q->mode));
@@ -2255,19 +2457,24 @@ char * rss_target;
 	set_attribute(sym,
 		l_string(std_cm,"Expire"),
 		l_string(std_cm,"-1"));
+
 	return List(sym,ret,-1);
 }
 
 void
-gvha_flush_pos(GVHA_FRAME_CACHE * c)
+gvha_flush_pos(GVHA_FRAME_CACHE * c,int * fp_flag)
 {
 GVHA_POS * p;
+
+ss_printf("FLUSH-POS %i\n",c->gf->id);
 	for ( ; c->pos ; ) {
 		p = c->pos;
 		c->pos = p->next;
 		d_f_ree(p->target);
 		d_f_ree(p);
 	}
+	if ( fp_flag )
+		*fp_flag = 0;
 }
 
 
@@ -2421,42 +2628,60 @@ ss_printf("\n");
 }
 
 void
-get_gvha_vct(GVHA_SET * g_set,GVHA_STATUS * sts,GVHA_VCT_SET * vs,int vct)
+get_gvha_vct(GVHA_SET * g_set,GVHA_STATUS * sts,GVHA_VCT_SET * vs,int vct,
+		char * vct_byname)
 {
 GVHA_COORDINATE * cc;
 URL u;
 	lock_task(gvha_lock);
-	for ( cc = g_set->coord_list ; cc ; cc = cc->next )
-		if ( cc->id == vct )
-			break;
-	if ( cc == 0 ) {
-		sts->status = GVHA_STS_NO_CRD_ID;
-		sts->err_message = List(n_get_symbol("ErrID"),
+	if ( vct ) {
+		for ( cc = g_set->coord_list ; cc ; cc = cc->next )
+			if ( cc->id == vct )
+				break;
+		if ( cc == 0 ) {
+			sts->status = GVHA_STS_NO_CRD_ID;
+			sts->err_message = List(n_get_symbol("ErrID"),
 					get_integer(vct,0),-1);
-		goto end;
-	}
-	vs->c_info = cc;
+			goto end;
+		}
+		vs->c_info = cc;
 	
-	get_url2(&u,cc->url);
-	vs->crd = search_resource_by_entry(&u);
-	if ( vs->crd == 0 ) {
-		sts->status = GVHA_STS_TARGET_CRD_IS_NOT_LOADED;
-		goto end;
+		get_url2(&u,cc->url);
+		vs->crd = search_resource_by_entry(&u);
+		free_url(&u);
+		if ( vs->crd == 0 ) {
+			sts->status = GVHA_STS_TARGET_CRD_IS_NOT_LOADED;
+			goto end;
+		}
+		get_url2(&u,cc->edit_target);
+		vs->vct = search_resource_by_entry(&u);
+		free_url(&u);
+		if ( vs->vct == 0 ) {
+			sts->status = GVHA_STS_TARGET_VCT_IS_NOT_LOADED;
+			goto end;
+		}
+		vs->env = new_env_pair(vector_env,gblisp_top_env0);
+		set_env(vs->env,l_string(std_cm,"__resource"),get_ptr(vs->vct,0));
 	}
-	free_url(&u);
-	get_url2(&u,cc->edit_target);
-	vs->vct = search_resource_by_entry(&u);
-	if ( vs->vct == 0 ) {
-		sts->status = GVHA_STS_TARGET_VCT_IS_NOT_LOADED;
-		goto end;
+	else {
+		vs->c_info = 0;
+		get_url2(&u,l_string(std_cm,vct_byname));
+		vs->crd = 0;
+		vs->vct = search_resource_by_entry(&u);
+		free_url(&u);
+		if ( vs->vct == 0 ) {
+			sts->status = GVHA_STS_TARGET_VCT_IS_NOT_LOADED;
+			goto end;
+		}
+		vs->env = new_env_pair(vector_env,gblisp_top_env0);
+		set_env(vs->env,l_string(std_cm,"__resource"),get_ptr(vs->vct,0));
 	}
-	vs->env = new_env_pair(vector_env,gblisp_top_env0);
-	set_env(vs->env,l_string(std_cm,"__resource"),get_ptr(vs->vct,0));
 end:
 	unlock_task(gvha_lock,"get_gvha_vct");
 }
 
 
+
 GB_POINT
 get_gvha_mark_point(GVHA_STATUS * sts,GVHA_FRAME_CACHE * c,GVHA_VCT_SET * vs,I_POINT display_ptr)
 {
@@ -2808,7 +3033,7 @@ ss_printf("S-5\n");
 			c->sts.flags = SF_WIDTH|SF_HEIGHT;
 			wf_set_status(c->gf,&c->sts);
 			locked_wf_status(c->gf,&c->sts);
-			gvha_flush_pos(c);
+			gvha_flush_pos(c,&flush_pos);
 			tmp = get_next_target(g_set,c,_dir,GVHA_MODE_RSS);
 			if ( tmp )
 				d_f_ree(tmp);
@@ -2835,11 +3060,12 @@ ss_printf("S-7\n");
 warp:
 
 #ifdef GVHA_PATH_DEBUG
-ss_printf("POS (%f %f) %f %f\n",
+ss_printf("POS (%f %f) %f %f %ls\n",
 q.pos.center.x,
 q.pos.center.y,
 q.pos.resolution,
-q.pos.rotate);
+q.pos.rotate,
+q.pos.target);
 #endif
 
 	flush_pos = 1;
@@ -2848,7 +3074,7 @@ q.pos.rotate);
 		&q.pos.center,
 		q.pos.resolution,
 		q.pos.rotate,
-		q.pos.target);
+		q.pos.target,0,1);
 change_layer:
 
 #ifdef GVHA_PATH_DEBUG
@@ -2941,9 +3167,17 @@ ss_printf("COMPLETE %i %i\n",c->all,c->ls.wf.dirty);
 	}
 finish2:
 
+#ifdef GVHA_PATH_DEBUG
+if ( c )
+ss_printf("MODE %i ==> %i\n",c->gf->id,q.mode);
+else
+ss_printf("MODE xx ==> %i\n",q.mode);
+#endif
+
 	switch ( q.mode ) {
 	case GVHA_MODE_MOVE:
 #ifdef GVHA_PATH_DEBUG
+if ( c )
 ss_printf("MOVE FM = %i\n",c->gf->id);
 #endif
 		rr.tl.x = rr.tl.y = 0;
@@ -2961,6 +3195,7 @@ ss_printf("MOVE FM = %i\n",c->gf->id);
 
 		locked_wf_status(c->gf,&c->sts);
 #ifdef GVHA_PATH_DEBUG
+if ( c )
 ss_printf("MOVE DIRTY = %i\n",c->sts.loading_status.dirty);
 #endif
 		sym = n_get_symbol("image/jpeg");
@@ -2985,8 +3220,11 @@ ss_printf("MOVE DIRTY = %i\n",c->sts.loading_status.dirty);
 		c->ls.wf = c->sts.loading_status;
 		loading_status_cal(c);
 
-		ret = return_rdf(g_set,&q,c,_dir);
-
+		ret = return_rdf(g_set,&q,c,_dir,&flush_pos);
+#ifdef GVHA_PATH_DEBUG
+if ( c )
+ss_printf("RSS INFO1 FM = %i\n",c->gf->id);
+#endif
 		wf_free_status(&c->c_sts);
 		break;
 	case GVHA_MODE_INFO2:
@@ -2994,6 +3232,7 @@ ss_printf("MOVE DIRTY = %i\n",c->sts.loading_status.dirty);
 		goto invalid_url;
 	case GVHA_MODE_RSS:
 	case GVHA_MODE_RSS_SHORT:
+	case GVHA_MODE_RSS_TINNY:
 		get_rc_loading_status(
 			(RADAR_CACHE*)c->gf->radar_ptr,&c->ls.rc);
 		wf_free_status(&c->sts);
@@ -3002,9 +3241,14 @@ ss_printf("MOVE DIRTY = %i\n",c->sts.loading_status.dirty);
 		c->ls.wf = c->sts.loading_status;
 		loading_status_cal(c);
 
-		ret = return_rdf(g_set,&q,c,_dir);
+		ret = return_rdf(g_set,&q,c,_dir,&flush_pos);
 #ifdef GVHA_PATH_DEBUG
+if ( c )
 ss_printf("RSS FM = %i\n",c->gf->id);
+/*
+print_sexp(s_stdout,ret,0);
+ss_printf("\n");
+*/
 #endif
 		break;
 	case GVHA_MODE_ECT_OPEN:
@@ -3016,7 +3260,7 @@ ss_printf("RSS FM = %i\n",c->gf->id);
 		break;
 	case GVHA_MODE_ECT_INSERT:
 		memset(&gs,0,sizeof(gs));
-		get_gvha_vct(g_set,&gs,&vs,q.e_vct);
+		get_gvha_vct(g_set,&gs,&vs,q.e_vct,q.e_vct_byname);
 		if ( gs.status )
 			goto insert_end;
 		m_ptr = get_gvha_mark_point(&gs,c,&vs,q.e_position);
@@ -3043,7 +3287,7 @@ ss_printf("RSS FM = %i\n",c->gf->id);
 		break;
 	case GVHA_MODE_ECT_DELETE:
 		memset(&gs,0,sizeof(gs));
-		get_gvha_vct(g_set,&gs,&vs,q.e_vct);
+		get_gvha_vct(g_set,&gs,&vs,q.e_vct,q.e_vct_byname);
 		if ( gs.status )
 			goto insert_end;
 		get_object_minrect2(&o_rect1,vs.vct,m_code);
@@ -3060,7 +3304,7 @@ ss_printf("RSS FM = %i\n",c->gf->id);
 
 	case GVHA_MODE_ECT_UPDATE:
 		memset(&gs,0,sizeof(gs));
-		get_gvha_vct(g_set,&gs,&vs,q.e_vct);
+		get_gvha_vct(g_set,&gs,&vs,q.e_vct,q.e_vct_byname);
 		if ( gs.status )
 			goto insert_end;
 
@@ -3085,7 +3329,7 @@ ss_printf("RSS FM = %i\n",c->gf->id);
 
 	case GVHA_MODE_ECT_MOVE:
 		memset(&gs,0,sizeof(gs));
-		get_gvha_vct(g_set,&gs,&vs,q.e_vct);
+		get_gvha_vct(g_set,&gs,&vs,q.e_vct,q.e_vct_byname);
 		if ( gs.status )
 			goto insert_end;
 		m_ptr = get_gvha_mark_point(&gs,c,&vs,q.e_position);
@@ -3112,7 +3356,7 @@ ss_printf("RSS FM = %i\n",c->gf->id);
 
 	case GVHA_MODE_ECT_SAVE:
 		memset(&gs,0,sizeof(gs));
-		get_gvha_vct(g_set,&gs,&vs,q.e_vct);
+		get_gvha_vct(g_set,&gs,&vs,q.e_vct,q.e_vct_byname);
 		if ( gs.status )
 			goto insert_end;
 		gvha_save(&gs,&vs);
@@ -3124,7 +3368,7 @@ ss_printf("RSS FM = %i\n",c->gf->id);
 		break;
 	case GVHA_MODE_ECT_OBJECT:
 		memset(&gs,0,sizeof(gs));
-		get_gvha_vct(g_set,&gs,&vs,q.e_vct);
+		get_gvha_vct(g_set,&gs,&vs,q.e_vct,q.e_vct_byname);
 		if ( gs.status )
 			goto insert_end;
 		gvha_object(&gs,&vs,q.e_objCode);
@@ -3133,9 +3377,15 @@ ss_printf("RSS FM = %i\n",c->gf->id);
 		er_panic("xl_GET");
 	}
 
+#ifdef GVHA_PATH_DEBUG
+if ( c )
+ss_printf("LAST-1 = %i\n",c->gf->id);
+else
+ss_printf("LAST-1 = xx\n");
+#endif
 	if ( c ) {
 		if ( flush_pos )
-			gvha_flush_pos(c);
+			gvha_flush_pos(c,&flush_pos);
 		if ( sts_load == 0 || flush_pos )
 			locked_wf_status(c->gf,&c->sts);
 		tmp = get_next_target(g_set,c,_dir,GVHA_MODE_RSS);
@@ -3153,6 +3403,12 @@ ss_printf("RSS FM = %i\n",c->gf->id);
 	else {
 		free_gvha_query(&q);
 	}
+#ifdef GVHA_PATH_DEBUG
+if ( c )
+ss_printf("LAST-2 = %i\n",c->gf->id);
+else
+ss_printf("LAST-2 = xx\n");
+#endif
 	
 	{
 	XL_SEXP * xml,*xml_sym;
@@ -3166,11 +3422,19 @@ ss_printf("RSS FM = %i\n",c->gf->id);
 		encode = get_sf_attribute(xml_sym->symbol.field,l_string(std_cm,"encoding"));
 		if ( encode ) {
 			cm = search_cm(n_string(std_cm,encode));
+			gc_push(0,0,"convert");
 			if ( cm )
 				ret = convert_code(ret,cm->main_code,CBF_ERR_QUESTION);
+			gc_pop(ret,gc_gb_sexp);
 		}
 	}
 rett:
+#ifdef GVHA_PATH_DEBUG
+if ( c )
+ss_printf("LAST-3 %i = %i\n",get_tid(),c->gf->id);
+else
+ss_printf("LAST-3 %i = xx\n",get_tid());
+#endif
 
 	return ret;
 
@@ -3210,7 +3474,10 @@ XL_SEXP *
 _xl_GET_call(
 	XLISP_ENV * e,XL_SEXP * s,XLISP_ENV * ae,XL_SYM_FIELD * sf)
 {
-	return _xl_GET(e,s,ae,sf,0);
+XL_SEXP * ret;
+
+	ret =  _xl_GET(e,s,ae,sf,0);
+	return ret;
 }
 
 XL_SEXP *
@@ -3219,13 +3486,16 @@ xl_GET(
 {
 XL_SEXP * sym;
 XL_SEXP * xl_Thread();
+XL_SEXP * ret;
+
 
 	sym = n_get_symbol("_GET_call");
 	sym->symbol.field = sf;
 	s = cons(sym,cdr(s));
-	return xl_Thread(e,
+	ret = xl_Thread(e,
 		List(n_get_symbol("Thread"),s,-1),
 		ae,0);
+	return ret;
 }
 
 
@@ -3240,6 +3510,7 @@ char * buf;
 L_CHAR * _dir;
 L_CHAR * _buf;
 int len;
+XL_SEXP * ret;
 
 	sym = n_get_symbol("_GET_call");
 	r = get_el(s,1);
@@ -3265,12 +3536,14 @@ int len;
 		}
 	}
 	s = cons(sym,0);
-	return xl_Thread(e,
+	ret = xl_Thread(e,
 		List(n_get_symbol("Thread"),s,-1),
 		ae,0);
+	return ret;
+
 type_missmatch:
 	return get_error(
-		s->h.file,
+	s->h.file,
 		s->h.line,
 		XLE_SEMANTICS_TYPE_MISSMATCH,
 		l_string(std_cm,"gbview/GET"),
