@@ -122,6 +122,7 @@ typedef struct matrix_token {
 
 	char *				c_file;
 	int				c_line;
+	
 } MATRIX_TOKEN;
 
 
@@ -175,6 +176,9 @@ typedef struct matrix_channel_info {
 #define MF_SEND		0x00000001
 #define MF_FILE		0x00000002
 #define MF_VISU		0x00000004
+
+#define MF_SEND_FILE	0x00000010
+#define MF_SEND_VISU	0x00000020
 	void *				default_data;
 	INTEGER64			default_data_fofs;
 } MATRIX_CHANNEL_INFO;
@@ -182,8 +186,9 @@ typedef struct matrix_channel_info {
 
 
 typedef struct matrix_channel {
-	void *			data;
-	INTEGER64		data_fofs;
+	void *				data;
+	INTEGER64			data_fofs;
+	struct mx_struct_block *	sb;
 } MATRIX_CHANNEL;
 
 typedef struct matrix_node_header {
@@ -230,6 +235,15 @@ typedef struct matrix_node {
 	INTEGER64		ring_fofs;
 	INTEGER64		dim_addr_fofs;
 	INTEGER64		dim_addr_size;
+	
+	/* for scan */
+	
+	char			scan_call[MI_MAX/3];
+
+	/* send opperation */
+
+	MATRIX_SEXP *		send_data;
+	MATRIX_SEXP *		send_data_delay;
 } MATRIX_NODE;
 
 #define MATRIX_NULLNODE		((INTEGER64)-1)
@@ -341,6 +355,7 @@ typedef struct matrix_data_type {
 #define MDT_STRING		0x0000000e
 #define MDT_SEXP		0x0000000f
 #define MDT_MAX			0x00000020
+	char *			type_name;
 	struct matrix_data_type * parent;
 	int			(*get_size)(struct matrix_data_type*,
 					void*);
@@ -389,7 +404,7 @@ typedef struct matrix_data_type {
 	void			(*endian_to_host)(void *);
 	void			(*convert_to_net)(struct matrix_data_type *,
 						RECORD_LIST64 * rlp,void *);
-	void *			(*convert_to_host)(struct matrix_data_type *,void *,int,void *);
+	void *			(*convert_to_host)(struct matrix_data_type *,void *,int size,int,void *,char*,int);
 
 	void * 			(*add_vv)(struct matrix_data_type *,
 					void *,void *,
@@ -522,6 +537,10 @@ typedef struct mx_cache_param {
 #define MXCS_DETAIL		1
 #define MXCS_RAND		2
 #define MXCS_STOP		3
+
+	/* vector read write param */
+	
+	int			vector_len;
 } MX_CACHE_PARAM;
 
 typedef struct mx_dim_code_list {
@@ -529,6 +548,41 @@ typedef struct mx_dim_code_list {
 	INTEGER64 *			dc;
 } MX_DIM_CODE_LIST;
 
+/* Structured Block */
+
+typedef struct mx_sb_new {
+	MATRIX *		m;
+	MATRIX_NODE *		n;
+	int			channel;
+} MX_SB_NEW;
+
+typedef struct mx_struct_block_tbl {
+	struct mx_struct_block *	(*new_sb)(MX_SB_NEW *,void*);
+	void			(*free_sb)(struct mx_struct_block * blk);
+	struct mx_struct_block *	(*copy_sb)(struct mx_struct_block *);
+	int			(*block2struct)(struct mx_struct_block*,MATRIX_ALLOC_BLOCK_PARAM*);
+	MATRIX_ALLOC_BLOCK_PARAM * (*struct2block)(struct mx_struct_block * blk);
+	int			(*operation)(int cmd,struct mx_struct_block * blk,void * param);
+} MX_STRUCT_BLOCK_TBL;
+
+typedef struct mx_struct_block {
+	MX_SB_NEW		n;
+	MX_STRUCT_BLOCK_TBL *	tbl;
+} MX_STRUCT_BLOCK;
+
+typedef struct mx_sb_list {
+	struct mx_sb_list *	next;
+	void *			work;
+	MX_STRUCT_BLOCK *	blk;
+} MX_SB_LIST;
+
+/* fundamental operation */
+#define SBOP_INSERT		1
+#define SBOP_DELETE		2
+#define SBOP_REFER		3
+#define SBOP_CALC		4
+
+
 #define ME_ERROR		(-100)
 #define ME_NO_NODE		(-101)
 #define ME_ERR_NODE		(-102)
@@ -555,8 +609,10 @@ typedef struct mx_dim_code_list {
 #define ME_MATRIX_ERR		(-121)
 #define ME_DATA_ACCESS		(-122)
 
-#define ME_CANNOT_OPEN_FILE	(-122)
-#define ME_DESTROY_FILE		(-123)
+#define ME_CANNOT_OPEN_FILE	(-123)
+#define ME_DESTROY_FILE		(-124)
+
+#define ME_EXIT			(-125)
 
 /* xlerror application code */
 
@@ -600,8 +656,8 @@ MATRIX
 
 void init_matrix();
 MATRIX_NODE * get_matrix_node(int * errp,MATRIX * m,
-			INTEGER64 * dim_code,int access,MATRIX_TOKEN*);
-INTEGER64 * get_dim_code_from_sexp(XL_SEXP * );
+			INTEGER64 * dim_code,int access,MATRIX_TOKEN*,XL_SEXP**);
+INTEGER64 * get_dim_code_from_sexp(MATRIX *,XL_SEXP * );
 XL_SEXP * get_channel_data(MATRIX_NODE * n,int ch,XL_SEXP * s);
 XL_SEXP * matrix_error(char*,XL_SEXP *,int code,XL_SEXP *);
 INTEGER64 * copy_dim_code(MATRIX * m,INTEGER64 * dim_code);
@@ -623,6 +679,7 @@ void matrix_trigger_access(MATRIX_NODE * n,int access,int clear_err);
 
 void init_mxCH(XLISP_ENV * env);
 void init_mxSet(XLISP_ENV * env);
+void init_mxSetSB(XLISP_ENV * env);
 void init_mxTrigger(XLISP_ENV * env);
 void init_mxAdd(XLISP_ENV * env);
 void init_mxDiff(XLISP_ENV * env);
@@ -630,6 +687,8 @@ void init_mxThinnedOut(XLISP_ENV * env);
 void init_mxCompressJPEG(XLISP_ENV * env);
 void init_mxUncompressJPEG(XLISP_ENV * env);
 void init_mxLoad(XLISP_ENV * env);
+void init_mxLoadSB(XLISP_ENV * env);
+void init_mxSaveSB(XLISP_ENV * env);
 void init_mxSave(XLISP_ENV * env);
 void init_mxFinish(XLISP_ENV * env);
 void init_mxUncompressOldFormat(XLISP_ENV * env);
@@ -801,7 +860,7 @@ void mxt_endian_vector_to_net(void *);
 void
 mxt_convert_basic_to_net(MATRIX_DATA_TYPE * tp,RECORD_LIST64 * rlp,void * d);
 void *
-mxt_convert_basic_to_host(MATRIX_DATA_TYPE * tp,void * d,int atype,void * at_work);
+mxt_convert_basic_to_host(MATRIX_DATA_TYPE * tp,void * d,int size,int atype,void * at_work,char*,int);
 void
 xx_insert_matrix_access(MATRIX_TOKEN * t,void (*func)(),char*,int);
 #define insert_matrix_access(t,f)	xx_insert_matrix_access(t,f,__FILE__,__LINE__)
@@ -822,6 +881,8 @@ void flush_mx_cache(MX_CACHE * c,int);
 int get_mx_cache(MX_CACHE * c,INTEGER64 * src);
 int write_mx_cache(MX_CACHE_PARAM * p);
 int read_mx_cache(MX_CACHE_PARAM * p);
+int read_mx_cache_vector(MX_CACHE_PARAM * p);
+int write_mx_cache_vector(MX_CACHE_PARAM * p);
 
 MX_DIM_CODE_LIST *
 mx_search_level_node(MATRIX * m,INTEGER64 * last_dc,int limit);
@@ -877,6 +938,54 @@ int mx_file_sync_create_node(MATRIX * m);
 void gc_mtx_block(MATRIX_ALLOC_BLOCK_PARAM * p);
 void
 xx_type_debug(char * msg,MATRIX_DATA_TYPE * tp,char * fn,int ln,unsigned int * tim);
+extern MATRIX_DATA_TYPE * mtx_type_tbl[MDT_MAX];
+
+typedef struct matrix_scan_t {
+	int			dir;
+#define MST_FIRST	0
+#define MST_AFTERLOAD	1
+#define MST_AFTERSRCH	2
+#define MST_LAST	3
+
+/* return types */
+#define MST_SRCH	0x00000001
+#define MST_1LOAD	0x00000002
+#define MST_2LOAD	0x00000004
+
+	INTEGER64 *		dim_code;
+	MATRIX *		m;
+	MATRIX_NODE *		n;
+	void *			channel_data;
+	void *			work;
+	int			channel;
+	int			dirty_flags;
+	
+	int			loading_type;
+#define SLT_NODE	1
+#define SLT_CHANNEL	2
+	int			gn_tree_node;
+	int			gn_create;
+} MATRIX_SCAN_T;
+int
+matrix_scan(MATRIX * m,int dir,int access,INTEGER64 * start,INTEGER64 * end,
+	int (*func)(MATRIX_SCAN_T *),void*work);
+int
+matrix_scan_force(MATRIX * m,int access,INTEGER64 * start,INTEGER64 * end,
+	int (*func)(MATRIX_SCAN_T *),void*work);
+XL_SEXP * encode_matrix_node(MATRIX_NODE *);
+int decode_matrix_node(MATRIX_NODE *,XL_SEXP *);
+void
+set_recordlist_code(RECORD_LIST64 * rl,CHAIN_LIST64 * ptr);
+void
+normalize_dim_code(MATRIX * m,INTEGER64 * dim);
+void
+set_dirty_file(MATRIX_NODE*);
+void
+_set_dirty_file(MATRIX_NODE * n);
+void
+set_dirty_file_sb(MX_STRUCT_BLOCK * b);
+
+PDB64 * get_mx_file_pfd(MATRIX * m);
 
 #define type_debug(msg,tp)	\
 	{				\
@@ -884,5 +993,16 @@ xx_type_debug(char * msg,MATRIX_DATA_TYPE * tp,char * fn,int ln,unsigned int * t
 		xx_type_debug(msg,tp,__FILE__,__LINE__,&tim); \
 	}
 
+void _finish_file_access(MATRIX * m,int);
+int _xx_start_file_access(MATRIX * m,int f,char *,int);
+int XX_start_file_access(MATRIX * m,char *,int);
+#define start_file_access(m)	\
+	XX_start_file_access(m,__FILE__,__LINE__)
+void finish_file_access(MATRIX * m,int);
+void finish_file_access_with_wait(MATRIX * m,int wait_key);
+MX_STRUCT_BLOCK * 
+get_channel_struct_block(MATRIX_NODE * n,int ch);
+
+
 #endif
 
