@@ -48,6 +48,11 @@
 #define LOADING_INTERVAL		5
 #define RADAR_ZONBIE_TIME		10
 
+#define INIT_RECOMMEND_INTERVAL		10
+#define RADAR_RI_RATE			0.1
+
+#define DISTORTION_LIMIT		3
+
 /* sethomestatus
 #define OPENING_TIMEOUT			5
 
@@ -64,6 +69,7 @@
 #define WT_RECT		1
 #define WT_CENTER	2
 #define WT_TARGET	3
+#define WT_WARP_POINT	4
 
 
 typedef struct lump_crd_node {
@@ -83,6 +89,10 @@ typedef struct radar_lump {
 	REAL1			reso_min;
 	REAL1			reso_max;
 	LUMP_CRD_NODE *		crd;
+//	INTEGER64		last_access_time;
+	float			recommend_interval;
+	int			q_len;
+	INTEGER64		invalid;
 } RADAR_LUMP;
 
 typedef struct radar_node {
@@ -120,6 +130,9 @@ typedef struct radar_crd {
 	LUMP_CRD_NODE *		lump;
 	int			lump_handling;
 	int *			q_thread_id;
+	
+	unsigned		invalid:1;
+	REAL1			distortion;
 } RADAR_CRD;
 
 typedef struct radar_matrix {
@@ -145,6 +158,8 @@ typedef struct t_load_to_beam {
 	REAL1			max;
 	L_CHAR *		base_url;
 	RADAR_LUMP *		rl;
+	INTEGER64		start_time;
+	int			que_len;
 } T_LOAD_TO_BEAM;
 
 typedef struct t_beam_to_resource {
@@ -186,6 +201,7 @@ typedef struct query_thread {
 	L_CHAR *		title;
 	int			flags;
 #define QTF_ACTIVE		0x00000001
+#define QTF_TEMP		0x00000002
 	int			mask;
 
 	L_CHAR *		url;
@@ -211,6 +227,44 @@ typedef struct q_thread_list {
 } Q_THREAD_LIST;
 
 
+
+typedef struct warp_point_layer {
+	struct warp_point_layer *	next;
+	L_CHAR *			entry;
+	int				flags;
+} WARP_POINT_LAYER;
+
+typedef struct warp_point {
+	int			type;
+#define WPT_SIMPLE		0x00000001
+#define WPT_CRD			0x00000002
+#define WPT_ONLY_ACTIVE		0x00000004
+	L_CHAR *		title;
+	INTEGER64		tracking_time;
+	QUERY_THREAD *		qth;
+	REAL1			resolution;
+	REAL1			rotate;
+	GB_POINT		center;
+	L_CHAR *		base;
+	L_CHAR *		lock_base;
+	WARP_POINT_LAYER *	layers;
+} WARP_POINT;
+
+typedef struct radar_history {
+	struct radar_history *	next;
+	struct radar_history *	prev;
+	struct radar_history *	acc_prev;
+/*
+	QUERY_THREAD *		qth;
+	REAL1			resolution;
+	REAL1			rotate;
+	GB_POINT		center;
+	L_CHAR *		base;
+	L_CHAR *		lock_base;
+*/
+	WARP_POINT *		wp;
+} RADAR_HISTORY;
+
 typedef struct radar_cache {
 	int			status;
 	GBVIEW_FLAME *		gf;
@@ -223,6 +277,9 @@ typedef struct radar_cache {
 	unsigned int		zonbie_time;
 	unsigned int		last_loading_time;
 	float			loading_interval;
+	float			loading_interval_past;
+	float			loading_interval_diff;
+	INTEGER64		loading_interval_sample_time;
 	int			matrix_count;
 	L_CHAR *		base_url;
 	int			base_url_lock;
@@ -250,6 +307,9 @@ typedef struct radar_cache {
 #define TMM_REMK	(1<<TM_REMK)
 #define TMM_INS		(1<<TM_INS)
 
+	RADAR_HISTORY *		history;
+	RADAR_HISTORY *		history_ptr;
+	Q_THREAD_LIST *		q_thread_tmp;
 	Q_THREAD_LIST *		q_thread_lst;
 //	XL_SEXP *		query;
 	char			query_mode;
@@ -263,6 +323,7 @@ typedef struct radar_cache {
 	REAL1			first_max;
 	GB_RECT			first_r;
 
+	int			lock_win_manage;
 	char			win_manage_task_wp;
 	char			loading_task_wp;
 	char			min_route_task_wp;
@@ -292,6 +353,18 @@ typedef struct radar_cache {
 
 	void 			(*condition_event)(QUERY_THREAD *,void*);
 	void *			condition_event_work;
+	
+	void			(*history_event)(struct radar_cache *,int direction,void*);
+#define HED_FORWARD	1
+#define HED_REVERSE	2
+	void *			history_event_work;
+	int			history_direction;
+
+	void			(*status_event)(struct radar_cache * rc_ptr,void*);
+	void *			status_event_work;
+
+	float			recommend_interval;
+	INTEGER64		last_access_loading;
 
 	struct radar_cache *	next;
 } RADAR_CACHE;
@@ -652,8 +725,32 @@ int rc_warp(
 	GB_POINT * center,
 	REAL1 resolution,
 	REAL1 rotate,
-	L_CHAR * target);
+	L_CHAR * target,
+	WARP_POINT*,
+	int);
 int get_rc_loading_status(RADAR_CACHE * rc_ptr,RC_LOADING_STATUS * rcs);
+int
+rc_next_history(RADAR_CACHE * rc_ptr);
+int
+rc_prev_history(RADAR_CACHE * rc_ptr);
+int
+rc_insert_history(RADAR_CACHE * rc_ptr);
+void
+rc_set_history_event(RADAR_CACHE * rc_ptr,void (*func)(RADAR_CACHE *,int,void*),void *work);
+void
+rc_set_status_event(RADAR_CACHE * rc_ptr,void (*func)(RADAR_CACHE *,void*),void *work);
+WARP_POINT *
+rc_get_warp_point(RADAR_CACHE *rc_ptr,int type);
+void
+free_warp_point_layer(WARP_POINT_LAYER * l);
+void
+free_warp_point(WARP_POINT * wp);
+WARP_POINT *
+sexp2wp(XL_SEXP * s);
+XL_SEXP *
+wp2sexp(WARP_POINT * wp);
+WARP_POINT *
+rc_copy_warp_point(WARP_POINT * wp);
 
 
 #endif
