@@ -256,11 +256,15 @@ typedef struct mark {
 #define AG_TB_TOP	0x00000000
 #define AG_TB_CENTER	0x00000001
 #define AG_TB_BOTTOM	0x00000002
+#define AG_TB_VALUE	0x00000003
 
 #define AG_LR_MASK	0x000000f0
 #define AG_LR_LEFT	0x00000000
 #define AG_LR_CENTER	0x00000010
 #define AG_LR_RIGHT	0x00000020
+#define AG_LR_VALUE	0x00000030
+	I_POINT		align_value;
+	L_CHAR *	anchor;
 	GB_POINT	pt;
 	GB_RECT		drect;
 } MARK;
@@ -386,6 +390,10 @@ typedef struct res_header {
 	int		cache_file_id;
 
 	int		draw_code;
+#ifdef USE_MATRIX
+	MATRIX *	mtx;
+	int		visu_channel;
+#endif
 } RES_HEADER;
 
 
@@ -400,14 +408,12 @@ typedef struct pixel_map_r64 {
 		'0'	CR 64 sqare RGB
 		'1'	CR 128 sqare RGB
 		'2'	CR 256 sqare RGB
+		'M'	MATRIX
 	*/
 	int		width[32];
 	int		height[32];
 	int		max_level;
 	TREE_CACHE	tc;
-#ifdef USE_MATRIX
-	MATRIX *	mtx;
-#endif
 	int		exit_lock;
 	int		loop_no;
 } PIXEL_MAP_R64;
@@ -450,7 +456,7 @@ typedef struct coordinate {
 	RING_TYPE	map_children;
 	RING_TYPE	map_parents;
 	OVERLAY *	ov;
-	REAL1		weight;
+	REAL1		_weight;
 	L_CHAR *	mode;
 	XL_SEXP *	initial_cmd;
 	XL_SEXP *	button_action;
@@ -585,6 +591,8 @@ typedef struct triangle {
 	AFFEN2D			a;
 	REAL1			resolution_rate;
 	GB_RECT			minrect;
+	
+	GB_INS_TRG_WORK		trg_w;
 } TRIANGLE;
 
 typedef struct triangle_set {
@@ -736,6 +744,7 @@ typedef struct draw_work {
 
 	int			data_request;
 	int			draw_code;
+	
 } DRAW_WORK;
 
 
@@ -878,6 +887,11 @@ typedef struct save_st_work {
 	STREAM *		st;
 } SAVE_ST_WORK;
 
+typedef struct map_convert_work {
+	GB_RECT *		sr;
+	int			sr_len;
+} MAP_CONVERT_WORK;
+
 typedef int (*GBV_FUNC)();
 
 #define GBF_DRAW	1
@@ -916,7 +930,8 @@ int browse_resource(RESOURCE *,BROWSE_WORK *);
 OBJ * search_obj(RESOURCE *,unsigned int);
 OBJ * new_object(RESOURCE *,int,unsigned int);
 
-void map_conv_reverse(RESOURCE *,GB_POINT * ,REAL1 * ,int);
+void xx_map_conv_reverse(RESOURCE *,MAP_CONVERT_WORK * w,GB_POINT * ,REAL1 * ,int,char*,int);
+#define map_conv_reverse(r,w,p,reso,size)	xx_map_conv_reverse(r,w,p,reso,size,__FILE__,__LINE__)
 void map_conv_forward(RESOURCE *,GB_POINT *,REAL1 *,int);
 void map_from_flame(MAP_HISTORY * mh,GB_POINT * ptr,REAL1 * reso,int size);
 void map_from_resource(MAP_HISTORY * mh,
@@ -989,6 +1004,7 @@ void set_lrw_err(LOAD_RESOURCE_WORK *,int);
 int lr_extend(RESOURCE * r,int (*)());
 
 XL_SEXP * load_meta(int ses,char * mode,L_CHAR * target);
+void load_meta_2(char * mode,L_CHAR * target,void (*func)(XL_SEXP *,L_CHAR *,void*),void * user_arg,int cache_flag);
 void _load_meta_list(int ses,char * mode,int data_flag,
 	int next_type,LOAD_RESOURCE_WORK * w);
 
@@ -1258,7 +1274,10 @@ int close_resource(RESOURCE * r);
 void set_resource_dirty(RESOURCE * r);
 void reset_resource_dirty(RESOURCE * r);
 int check_resource_dirty(RESOURCE * r);
-
+int
+map_history_length(MAP_HISTORY * mh);
+BIB_LIST *
+gv_get_biblist(L_CHAR * url);
 
 
 /* Channels of old matrix */
