@@ -51,7 +51,7 @@
 
 int curret_check(XL_SEXP *);
 
-unsigned int hash_key(L_CHAR*);
+unsigned int hash_key(L_CHAR*,int);
 
 void print_sexp(STREAM *,XL_SEXP * ,int);
 int e_print_sexp(STREAM *,int *,XL_SEXP * ,int);
@@ -111,7 +111,7 @@ XLISP_ENV * new_env_pair(XLISP_ENV *,XLISP_ENV *);
 void init_gblisp();
 XL_SEXP * eval(XLISP_ENV * env,XL_SEXP * sym);
 XL_SEXP *
-binary(BINARY_TABLE bt[XLT_MAX][XLT_MAX],XLISP_ENV *,XL_SEXP *,XL_SEXP *);
+binary(BINARY_TABLE bt[XLT_MAX][XLT_MAX],XLISP_ENV *,XL_SEXP *,XL_SEXP *,XL_FILE*,int);
 ELEMENT *
 search_env(XLISP_ENV * env,L_CHAR * sym);
 void set_env(XLISP_ENV * env,L_CHAR * sym,XL_SEXP *);
@@ -538,6 +538,7 @@ L_CHAR * get_sys_path(char *c);
 XL_SEXP * format_HTML(XL_SEXP * inp);
 XL_SEXP *
 xl_format_HTML(XLISP_ENV * env,XL_SEXP * s,XLISP_ENV* arg_env,XL_SYM_FIELD * sf);
+void xl_set_cm(XL_SEXP * s,CODE_METHOD * cm);
 
 
 void init_SetDirectory(XLISP_ENV * env);
