@@ -18,6 +18,7 @@
 
 
 
+
 #define OPEN_FILESPACE
 
 #include	"machine/include.h"
@@ -29,7 +30,7 @@
 #include	"favt64.h"
 
 INTEGER64
-fa_write64(int fd,void * d,U_INTEGER64 len);
+fa_write64(PDB64 * p,void * d,U_INTEGER64 len);
 U_INTEGER64
 fs_raw_write64(PDB64 * p,void * d,int flags);
 void change_endian_freelist64_to_net(PN64_FREELIST * f);
@@ -47,6 +48,8 @@ void truncate_check64(PDB64 * p,U_INTEGER64 length);
 int check_freelist64(PDB64 * p);
 int check_filespace64(PDB64 * p);
 
+//U_INTEGER64 debug_target_fofs=(U_INTEGER64)56;
+U_INTEGER64 debug_target_fofs=(U_INTEGER64)-1;
 
 void
 check_pdb64(PDB64 * p)
@@ -57,16 +60,188 @@ check_pdb64(PDB64 * p)
 		er_panic("check_pdb64");
 }
 
+void
+check_access_write(int fd,U_INTEGER64 length)
+{
+INTEGER64 start;
+
+	start = u_lseek64(fd,0,SEEK_CUR);
+	if ( start < 0 )
+		er_panic("CAW");
+	if ( start <= debug_target_fofs && start+length > debug_target_fofs )
+		er_panic("STOP");
+}
+
+int
+pdb64_debug_cmp(void * a,void * b)
+{
+PDB64_DEBUG * ap,*bp;
+	ap = (PDB64_DEBUG*)a;
+	bp = (PDB64_DEBUG*)b;
+	if ( ap->fofs < bp->fofs )
+		return -1;
+	if ( ap->fofs > bp->fofs )
+		return 1;
+	return 0;
+}
+
+
+int
+debug_pdb64_bound_cmp(PDB64_DEBUG * d1,INTEGER64 * d2,void * w)
+{
+	if ( d1->fofs < *d2 )
+		return -1;
+	if ( d1->fofs > *d2 )
+		return 1;
+	return 0;
+}
+
+int
+debug_pdb64_list(PDB64 * p,int cmd,INTEGER64 fofs,INTEGER64 size)
+{
+AVT_NODE * a,*a2;
+U_INTEGER64 fofs2;
+PDB64_DEBUG d, * dp;
+PN64_HEADER * h;
+INTEGER64 _size;
+AVT_BOUND_LIST * b;
+int er;
+INTEGER64 b1,b2;
+INTEGER64 ofs_buf;
+	if ( (p->flags & PF_DEBUG) == 0 )
+		return 0;
+	if ( cmd & DT_POS ) {
+		fofs = u_lseek64(p->fid,0,SEEK_CUR);
+	}
+//ss_printf(">> %p >> %x:" I64_FORMAT " " I64_FORMAT "\n",p,cmd,fofs,size);
+	switch ( cmd & DT_CMD ) {
+	case DT_SEARCH:
+		d.fofs = fofs;
+		d.size = size;
+		a = avt_search(p->pdb64_debug_root,&d,pdb64_debug_cmp);
+		if ( a == 0 )
+			return -1;
+		else	return 0;
+	case DT_DP_SEARCH:
+		d.fofs = fofs;
+		d.size = size;
+		a = avt_search(p->pdb64_debug_root,&d,pdb64_debug_cmp);
+		if ( a )
+			return 0;
+		fofs2 = 0;
+		for ( _size = 10 ; _size < fofs ; _size = _size*2 ) {
+			b1 = fofs - _size;
+			b2 = fofs;
+			b = avt_bound_search(&er,p->pdb64_debug_root,&b1,&b2,1,
+					debug_pdb64_bound_cmp,0);
+			if ( b == 0 )
+				continue;
+			dp = b->node->data;
+			fofs2 = dp->fofs;
+			d_f_ree(b);
+			break;
+		}
+ss_printf(">>> DP SEARCH %lli %lli %lli\n",fofs,_size,fofs2);
+		ofs_buf = u_lseek64(p->fid,0,SEEK_CUR);
+		for ( ; fofs2 < fofs ; ) {
+			p->flags |= PF_NEST_DEBUG;
+			h = get_file_record64(&fofs2,p,0,0);
+			p->flags &= ~PF_NEST_DEBUG;
+			if ( h == 0 )
+				break;
+			change_endian_i(h->type);
+			change_endian_i(h->size);
+//			ss_printf(I64_FORMAT " H(%x " I64_FORMAT ")\n",fofs,h->type,h->size);
+			fofs2 += PHYS_SIZE(h->size);
+			_size = h->size;
+			d_f_ree(h);
+		}
+		u_lseek64(p->fid,ofs_buf,SEEK_SET);
+		if ( fofs == fofs2 ) {
+			dp = d_alloc(sizeof(*dp));
+			a = d_alloc(sizeof(*a));
+			a->data = dp;
+			dp->size = _size;
+			dp->fofs = fofs;
+			a2 = avt_insert(&p->pdb64_debug_root,a,pdb64_debug_cmp);
+			return 0;
+		}
+		return -1;
+	case DT_INSERT:
+/*
+if ( fofs == 212 )
+er_panic("A");
+*/
+		dp = d_alloc(sizeof(*dp));
+		a = d_alloc(sizeof(*a));
+		a->data = dp;
+		dp->size = size;
+		dp->fofs = fofs;
+		a2 = avt_insert(&p->pdb64_debug_root,a,pdb64_debug_cmp);
+		if ( a2 != a ) {
+			dp = a2->data;
+			if ( size >= 0 )
+				dp->size = size;
+			d_f_ree(a->data);
+			d_f_ree(a);
+			return -1;
+		}
+		return 0;
+	case DT_DELETE:
+		d.fofs = fofs;
+		d.size = size;
+		a = avt_delete(&p->pdb64_debug_root,dp,pdb64_debug_cmp);
+		if ( a ) {
+			d_f_ree(a->data);
+			d_f_ree(a);
+			return 0;
+		}
+		return -1;
+	case DT_ALL_DELETE:
+		for (;  p->pdb64_debug_root ; ) {
+			dp = p->pdb64_debug_root->data;
+			a = avt_delete(&p->pdb64_debug_root,dp,pdb64_debug_cmp);
+			d_f_ree(a);
+			d_f_ree(dp);
+		}
+		return 0;
+	case DT_BOUNDARY:
+
+		b1 = fofs + 1;
+		b2 = fofs + size - 1;
+		b = avt_bound_search(&er,p->pdb64_debug_root,&b1,&b2,1,
+				debug_pdb64_bound_cmp,0);
+		if ( b )  {
+			dp = b->node->data;
+			printf("B " I64_FORMAT " " I64_FORMAT " " I64_FORMAT "\n",
+				fofs,fofs + size,dp->fofs);
+			er_panic("debug_pdb64_list");
+		}
+		return 0;
+	default:
+		er_panic("debug_pdb64_list");
+	}
+	return 0;
+}
+
 
 INTEGER64
-fa_write64(int fd,void * d,U_INTEGER64 len)
+fa_write64(PDB64 * p,void * d,U_INTEGER64 len)
 {
 char * dp;
 int er;
 int ret;
 int _s;
-	ret = len;
+int fd;
+U_INTEGER64 _len;
+	fd = p->fid;
 
+	ret = len;
+	_len = len;
+
+	if ( debug_pdb64_list(p,DT_DP_SEARCH|DT_POS,0,0) < 0 )
+		er_panic("fa_write64");
+	debug_pdb64_list(p,DT_BOUNDARY|DT_POS,0,_len);
 	for ( dp = (char*)d ; len > 0 ; ) {
 		if ( len > 0x10000 )
 			_s = 0x10000;
@@ -77,18 +252,31 @@ int _s;
 		len -= er;
 		dp += er;
 	}
+	
+	
 	if ( len == 0 )
 		return ret;
 	else	return -1;
 }
 
 INTEGER64
-fa_write64_header(int fd,void * d,U_INTEGER64 len)
+fa_write64_header(PDB64 * p,void * d,U_INTEGER64 len)
 {
 PN64_HEADER h;
 PHYS_PN64_HEADER ph;
 int h_size;
 INTEGER64 ret;
+int fd;
+int flags;
+INTEGER64 st_ofs;
+
+	st_ofs = u_lseek64(p->fid,0,SEEK_CUR);
+
+	fd = p->fid;
+
+	if ( debug_pdb64_list(p,DT_DP_SEARCH|DT_POS,0,0) < 0 )
+		er_panic("fa_write64");
+
 	h = *(PN64_HEADER*)d;
 	change_endian_header64(&h);
 
@@ -115,13 +303,18 @@ er_panic("???");
 
 	change_endian_phys_header64_to_net(&ph);
 
-	ret = fa_write64(fd,&ph,h_size);
+	ret = fa_write64(p,&ph,h_size);
 	if ( ret < 0 )
 		return ret;
+	flags = p->flags & PF_DEBUG;
+	p->flags &= ~PF_DEBUG;
 	len -= sizeof(PN64_HEADER);
-	if ( len )
-		return ret + fa_write64(fd,((PN64_HEADER*)d)+1,len);
-	else	return ret;
+	if ( len >= 0 )
+		ret = ret + fa_write64(p,((PN64_HEADER*)d)+1,len);
+	p->flags |= flags;
+	
+	debug_pdb64_list(p,DT_BOUNDARY,st_ofs,len+h_size);
+	return ret;
 }
 
 
@@ -136,30 +329,39 @@ U_INTEGER64 size;
 char dummy[DUMMY_SIZE];
 int s;
 int er,len;
-U_INTEGER64 offset;
+U_INTEGER64 offset,last_offset;
+int pflags;
 
 	offset = u_lseek64(p->fid,0,SEEK_END);
 	h = d;
 	hh = *h;
 	change_endian_header64(&hh);
 	size = hh.size;
+	debug_pdb64_list(p,DT_INSERT,offset,size);
 	switch ( flags &(FSF_ONLYHEADER|FSF_RESERVE_SP) ) {
 	case 0:
-		fa_write64_header(p->fid,d,size);
+		fa_write64_header(p,d,size);
 		break;
 	case FSF_ONLYHEADER:
-		fa_write64_header(p->fid,d,sizeof(*h));
+		fa_write64_header(p,d,sizeof(*h));
 		break;
 	case FSF_RESERVE_SP:
 	case FSF_RESERVE_SP|FSF_ONLYHEADER:
-		fa_write64_header(p->fid,d,sizeof(*h));
+		fa_write64_header(p,d,sizeof(*h));
+		pflags = p->flags & PF_DEBUG;
+		p->flags &= ~PF_DEBUG;
 		for ( s = size - sizeof(*h) ; s > 0 ; ) {
 			if ( s > DUMMY_SIZE )
 				len = DUMMY_SIZE;
 			else	len = s;
-			er = fa_write64(p->fid,dummy,len);
+			er = fa_write64(p,dummy,len);
 			s -= er;
 		}
+		p->flags |= pflags;
+		last_offset = u_lseek64(p->fid,0,SEEK_CUR);
+
+		debug_pdb64_list(p,DT_BOUNDARY,offset,last_offset - offset);
+		
 		break;
 	}
 	return offset;
@@ -205,14 +407,24 @@ INTEGER64 read_size;
 		fofs = 0;
 		f = 1;
 	}
+
+	if ( (p->flags & PF_NEST_DEBUG) && (debug_pdb64_list(p,DT_DP_SEARCH,fofs,0) < 0) )
+		er_panic("get_file_record64 position");
+
 	length = u_lseek64(p->fid,0,SEEK_END);
 	if ( length < 0 ) {
+ss_printf("get_file_record64-1 = " I64_FORMAT "\n",length);
 		return 0;
 	}
 	for ( ; ; ) {
 		if ( fofs >= length ) {
+ss_printf("get_file_record64-2 = " I64_FORMAT " " I64_FORMAT "\n",
+length,fofs);
 			return 0;
 		}
+
+		debug_pdb64_list(p,DT_INSERT,fofs,0);
+
 		er = u_lseek64(p->fid,fofs,SEEK_SET);
 		if ( er < 0 )
 			er_panic("get_file_record64(1)");
@@ -224,8 +436,10 @@ INTEGER64 read_size;
 			er_panic("get_file_record64(2)");
 		h16_size = ph.h16.size;
 		change_endian(h16_size);
-		if ( h16_size < sizeof(PN_HEADER) )
+		if ( h16_size < sizeof(PN_HEADER) ) {
+			ss_printf("%i %x %x " I64_FORMAT "\n",(int)h16_size,(int)ph.h16.size,(int)ph.h16.type,(INTEGER64)fofs);
 		  	er_panic("data error");
+		}
 		switch ( h16_size ) {
 		case PHYS_SIZE32:
 			er = u_read(p->fid,&ph.h32.size,sizeof(int));
@@ -299,10 +513,15 @@ INTEGER64 length;
 PN64_FILE_HEADER h;
 PN64_FREELIST	f, * ff;
 FREELIST64 * fp;
+U_INTEGER64 fofs;
 	p = open_pdb64(filename,oflags,mode,flags&PF_PDB_MASK);
 	if ( p == 0 ) {
 		return 0;
 	}
+	p->pdb64_debug_root = 0;
+//p->flags |= PF_DEBUG;
+	debug_pdb64_list(p,DT_INSERT,0,0);
+		
 	length = u_lseek64(p->fid,0,SEEK_END);
 	if ( length == 0 ) {
 		h.h.size = sizeof(h);
@@ -315,6 +534,9 @@ FREELIST64 * fp;
 		h.linear = 0;
 		h.linear_length = 0;
 		change_endian_file_header64(&h);
+		
+		debug_pdb64_list(p,DT_INSERT,0,sizeof(h));
+		
 		fs_raw_write64(p,&h,0);
 		if ( flags & PF_USEFREELIST ) {
 			f.h.size = sizeof(f);
@@ -330,12 +552,13 @@ FREELIST64 * fp;
 		}
 	}
 	else {
-		ff = get_file_record64(0,p,PNT_FREELIST_HEADER,0);
+		fofs = 0;
+		ff = get_file_record64(&fofs,p,PNT_FREELIST_HEADER,0);
 		if ( ff ) {
 			fp = d_alloc(sizeof(*fp));
 			memcpy(&fp->f,ff,sizeof(*ff));
 			fp->next = 0;
-			fp->fofs = sizeof(h);
+			fp->fofs = fofs;
 			d_f_ree(ff);
 			change_endian_freelist64_to_host(&fp->f);
 			p->freelist_work = fp;
@@ -364,10 +587,12 @@ FREELIST64 * f2;
 void
 close_filespace64(PDB64 * p)
 {
+
 	if ( p->freelist_work )
 		free_freelist64(p->freelist_work);
 	p->freelist_work = 0;
 	flush_favt64_cache(p);
+	debug_pdb64_list(p,DT_ALL_DELETE,0,0);
 	close_pdb64(p);
 }
 
@@ -385,6 +610,10 @@ PN64_FREELIST * _f;
 
 	_f = get_file_record64(&fofs,p,0,sizeof(PN64_FREELIST));
 	change_endian_freelist64_to_host(_f);
+
+if ( fofs == 44 )
+ss_printf("LOAD FREELIST64\n");
+
 	*f = *_f;
 	d_f_ree(_f);
 }
@@ -397,7 +626,10 @@ PN64_FREELIST buf;
 	buf = *f;
 	change_endian_freelist64_to_net(&buf);
 	u_lseek64(p->fid,fofs,SEEK_SET);
-	er = fa_write64_header(p->fid,&buf,sizeof(buf));
+
+	debug_pdb64_list(p,DT_INSERT,fofs,sizeof(buf));
+
+	er = fa_write64_header(p,&buf,sizeof(buf));
 	if ( er < 0 )
 		er_panic("save_freelist");
 }
@@ -455,6 +687,7 @@ PN64_FREELIST buf;
 	if ( b->f.next == 0 )
 		return -1;
 	delete_fofs = b->f.next;
+	
 	if ( b->next ) {
 		d = b->next;
 		b->f.next = d->f.next;
@@ -464,7 +697,9 @@ PN64_FREELIST buf;
 	else {
 		load_freelist64(p,&buf,delete_fofs);
 		b->f.next = buf.next;
+	
 	}
+	debug_pdb64_list(p,DT_DELETE,delete_fofs,0);
 
 	save_freelist64(p,&b->f,b->fofs);
 
@@ -508,6 +743,9 @@ fit_ok:
 
 check_freelist64(p);
 
+//ss_printf("ALLOC " I64_FORMAT " " I64_FORMAT "\n",ret,h->size);
+	debug_pdb64_list(p,DT_INSERT,ret,0);
+
 	return ret;
 large_ok:
 
@@ -525,6 +763,9 @@ large_ok:
 
 check_freelist64(p);
 
+	debug_pdb64_list(p,DT_INSERT,ret,0);
+
+//ss_printf("ALLOC " I64_FORMAT " " I64_FORMAT "\n",ret,h->size);
 	return ret;
 nothing:
 
@@ -536,6 +777,9 @@ nothing:
 
 check_freelist64(p);
 
+	debug_pdb64_list(p,DT_INSERT,ret,0);
+
+//ss_printf("ALLOC " I64_FORMAT " " I64_FORMAT "\n",ret,h->size);
 	return ret;
 }
 
@@ -565,6 +809,7 @@ INTEGER64 fofs;
 	ftruncate64(p->fid,fofs);
 }
 
+
 void
 free_filespace64(PDB64 * p,U_INTEGER64 fofs)
 {
@@ -572,8 +817,10 @@ PN64_FREELIST f;
 FREELIST64 * fp,* fp1,* prev;
 U_INTEGER64 length;
 
+
 check_freelist64(p);
 
+//ss_printf("FREE FILESPACE " I64_FORMAT "\n",fofs);
 	if ( fofs == 0 )
 		er_panic("free_filespace64");
 	load_freelist64(p,&f,fofs);
@@ -589,9 +836,11 @@ check_freelist64(p);
 	fp = p->freelist_work;
 	if ( fp == 0 )
 		er_panic("free_filespace(1)");
+
 	fp1 = next_freelist64(p,fp);
 	if ( fp1 == 0 )
 		goto only_one;
+
 	if ( fp1->fofs < fofs )
 		goto large;
 	prev = fp;
@@ -606,11 +855,15 @@ check_freelist64(p);
 	}
 next_nothing:
 
+
+
 	if ( fofs + PHYS_SIZE(f.h.size) == fp->fofs )
 		goto stack_on;
 	goto only_one;
 three_type:
 
+
+
 	if ( fofs + PHYS_SIZE(f.h.size) == fp->fofs ) {
 		if ( fp1->fofs + PHYS_SIZE(fp1->f.h.size) == fofs )
 			goto sandwitch;
@@ -621,6 +874,8 @@ three_type:
 	else	goto only_one;
 large:
 
+
+
 	if ( fp1->fofs + PHYS_SIZE(fp1->f.h.size) == fofs )
 		goto hang_on;
 	else if ( fp1->fofs + PHYS_SIZE(fp1->f.h.size) > fofs ) {
@@ -637,6 +892,8 @@ large:
 
 only_one:
 
+
+
 	insert_freelist64(p,fp,fofs,f.h.size);
 
 check_freelist64(p);
@@ -645,6 +902,7 @@ check_freelist64(p);
 	return;
 hang_on:
 
+
 	fp1->f.h.size = LOGI_SIZE(PHYS_SIZE(fp1->f.h.size) + PHYS_SIZE(f.h.size));
 	save_freelist64(p,&fp1->f,fp1->fofs);
 
@@ -654,6 +912,8 @@ check_freelist64(p);
 	return;
 stack_on:
 
+
+
 	fp->f.h.size = LOGI_SIZE(PHYS_SIZE(fp->f.h.size) + PHYS_SIZE(f.h.size));
 	fp->fofs  = fofs;
 	save_freelist64(p,&fp->f,fp->fofs);
@@ -666,6 +926,7 @@ check_freelist64(p);
 	return;
 sandwitch:
 
+
 	f.h.size = LOGI_SIZE(PHYS_SIZE(fp1->f.h.size) + PHYS_SIZE(fp->f.h.size)
 			+ PHYS_SIZE(f.h.size));
 	if ( delete_freelist64(0,p,prev) < 0 )
@@ -675,7 +936,6 @@ sandwitch:
 
 check_freelist64(p);
 
-
 	return;
 }
 
@@ -839,7 +1099,11 @@ unsigned short s16;
 int _s;
 char * ptr;
 
+	if ( debug_pdb64_list(p,DT_DP_SEARCH,fofs,0) < 0 )
+		er_panic("read_filespace");
+	
 	u_lseek64(p->fid,fofs,SEEK_SET);
+	
 	er = u_read(p->fid,&hh,sizeof(PN_HEADER));
 	if ( er < sizeof(PN_HEADER) )
 		goto err;
@@ -912,6 +1176,9 @@ U_INTEGER64 size,_size;
 PHYS_PN64_HEADER hh;
 unsigned short s16;
 
+	if ( debug_pdb64_list(p,DT_DP_SEARCH,fofs,0) < 0 )
+		er_panic("read_filespace");
+	
 	u_lseek64(p->fid,fofs,SEEK_SET);
 	er = u_read(p->fid,&hh,sizeof(PN_HEADER));
 	if ( er < sizeof(PN_HEADER) )
@@ -967,7 +1234,7 @@ U_INTEGER64 size;
 	size = a->h.size;
 	change_endian(size);
 	u_lseek64(p->fid,a->a.fofs,SEEK_SET);
-	return fa_write64_header(p->fid,&a->h,size);
+	return fa_write64_header(p,&a->h,size);
 }
 
 void *
@@ -980,6 +1247,9 @@ U_INTEGER64 size,_size;
 PHYS_PN64_HEADER hh;
 unsigned short s16;
 
+	if ( debug_pdb64_list(p,DT_DP_SEARCH,fofs,0) < 0 )
+		er_panic("read_filespace");
+	
 	u_lseek64(p->fid,fofs,SEEK_SET);
 	er = u_read(p->fid,&hh,sizeof(PN_HEADER));
 	if ( er < sizeof(PN_HEADER) )
@@ -1037,6 +1307,9 @@ int _s;
 char * ptr;
 int err_code;
 
+	if ( debug_pdb64_list(p,DT_DP_SEARCH,fofs,0) < 0 )
+		er_panic("read_filespace");
+	
 	u_lseek64(p->fid,fofs,SEEK_SET);
 	er = u_read(p->fid,&hh,sizeof(PN_HEADER));
 	err_code = 1;
@@ -1118,7 +1391,7 @@ err:
 }
 
 int
-write_filespace64f(PDB64 * p,int fofs,void * h)
+write_filespace64f(PDB64 * p,U_INTEGER64 fofs,void * h)
 {
 U_INTEGER64 size;
 PN64_HEADER * a;
@@ -1126,6 +1399,60 @@ PN64_HEADER * a;
 	size = a->size;
 	change_endian(size);
 	u_lseek64(p->fid,fofs,SEEK_SET);
-	return fa_write64_header(p->fid,h,size);
+	return fa_write64_header(p,h,size);
 }
 
+
+int test_file64_count;
+
+void
+test_file64(char * file,PDB64 * _p,INTEGER64 limit)
+{
+PDB64 * p;
+U_INTEGER64 fofs;
+char * encoding;
+PN64_HEADER * h;
+
+/*
+test_file64_count ++;
+
+if ( test_file64_count < 336000 )
+return;
+limit = 10000000;
+*/
+	if ( file ) {
+		encoding = "UCS4";
+		p = open_filespace64(
+			file,
+			O_RDONLY,0644,
+			PF_USEFREELIST,FT_RT_CACHE,encoding);
+		_p = 0;
+
+
+	}
+	else {
+		p = _p;
+	}
+	if ( p == 0 ) {
+		er_panic("test_file64\n");
+	}
+	fofs = 0;
+	for ( ; ; ) {
+		if ( limit > 0 && fofs >= limit )
+			break;
+		h = get_file_record64(&fofs,p,0,0);
+		if ( h == 0 )
+			break;
+		change_endian_i(h->type);
+		change_endian_i(h->size);
+		ss_printf(I64_FORMAT " H(%x " I64_FORMAT ")\n",fofs,h->type,h->size);
+		fofs += PHYS_SIZE(h->size);
+		d_f_ree(h);
+	}
+	if ( _p == 0 )
+		close_filespace64(p);
+	ss_printf("FINISH\n");
+}
+
+
+
