@@ -66,13 +66,6 @@ int mx_dim_code_cmp(MX_DIM_CODE * dc1,MX_DIM_CODE * dc2,MATRIX*);
 int save_subheader(MATRIX * m,int);
 int load_subheader(MATRIX * m,int);
 void mx_file_lock_event(int seq,MATRIX * m);
-void _finish_file_access(MATRIX * m,int);
-int _xx_start_file_access(MATRIX * m,int f,char *,int);
-int XX_start_file_access(MATRIX * m,char *,int);
-#define start_file_access(m)	\
-	XX_start_file_access(m,__FILE__,__LINE__)
-void finish_file_access(MATRIX * m,int);
-void finish_file_access_with_wait(MATRIX * m,int wait_key);
 void _close_matrix_file(MATRIX * m);
 void setup_mx_cache_ring(MATRIX * m,INTEGER64 ring,INTEGER64 node);
 void mx_cache_purge(MATRIX * m,INTEGER64 cr_fofs);
@@ -140,6 +133,10 @@ retry:
 		if ( mx_open_files_cnt < MAX_OPEN_FILES_FOR_MX ) {
 			if ( get_xltime() - m1->last_access < MLOCK_LIFE_TIME )
 				continue;
+			if ( m1->wait_token )
+				continue;
+			if ( m1->token_cnt )
+				continue;
 		}
 		m1->access_lock ++;
 		unlock_task(matrix_lock,"mx_file_gc");
@@ -623,6 +620,7 @@ MATRIX_DATA_TYPE * tp;
 	h = rl->data;
 
 	w->header_fofs = alloc_filespace64(w->pdb_fd,rl->data);
+ss_printf("H-1 " I64_FORMAT "\n",h->h.size);
 
 	for ( i = 0 ; i < m->p.channel_nos ; i ++ ) {
 		ci = &m->channel_info[i];
@@ -661,10 +659,8 @@ MATRIX_DATA_TYPE * tp;
 
 	setup_recordlist64(rl);
 	h = rl->data;
-
-printf("mx_file %llx %p %p\n",h->h.size,h,rl->data);
+ss_printf("H-LAST " I64_FORMAT "\n",h->h.size);
 	change_endian_header64(&h->h);
-printf("mx_file %llx %p %p\n",h->h.size,h,rl->data);
 	change_endian(h->channel_nos);
 	change_endian(h->dim);
 	change_endian(h->flags);
@@ -680,6 +676,32 @@ printf("mx_file %llx %p %p\n",h->h.size,h,rl->data);
 	return 0;
 }
 
+
+void
+matrix_test(MATRIX * m)
+{
+MX_FILE_WORK * w;
+PN64_MX_HEADER * h;
+int err;
+U_INTEGER64 ofs;
+
+
+	err = start_file_access(m);
+	if ( err < 0 ) {
+ss_printf("load_matrix_header-1 %i %p %p\n",err,m->file_work,m);
+		return;
+	}
+	w = m->file_work;
+
+	ofs = 0;
+	h = get_file_record64((U_INTEGER64*)&ofs,
+		w->pdb_fd,PNT_MX_HEADER,0);
+
+	if ( h )
+		d_f_ree(h);
+	finish_file_access(m,0);
+}
+
 int
 load_matrix_header(MATRIX * m,int oflags,int mode)
 {
@@ -697,10 +719,11 @@ MATRIX_DATA_TYPE * tp;
 int err;
 
 	if ( m->filename == 0 )
-		return -1;
+		return ME_ERROR;
 
 	w = d_alloc(sizeof(*w));
 	memset(w,0,sizeof(*w));
+
 	m->file_work = w;
 	w->oflags = oflags;
 
@@ -852,8 +875,8 @@ MX_FILE_WORK * w;
 		set_recordlist_chain64
 			(rl,&size,sizeof(PN64_HEADER)+sizeof(INTEGER64)
 					- rl->header.size,0);
+		setup_recordlist64(rl);
 	}
-	setup_recordlist64(rl);
 
 	if( fofs == 0 ) {
 	alloc:
@@ -900,7 +923,6 @@ PN64_MX_DATA_2 * mx;
 
 	if ( start_file_access(m) < 0 )
 		return -1;
-
 	w = m->file_work;
 
 	rl = new_recordlist64(
@@ -913,7 +935,7 @@ PN64_MX_DATA_2 * mx;
 	change_endian(mx->next_fofs);
 	change_endian(mx->id);
 
-	(tp->convert_to_net)(tp,rl,target);
+	(*tp->convert_to_net)(tp,rl,target);
 
 	setup_recordlist64(rl);
 	mx = (PN64_MX_DATA_2*)rl->data;
@@ -970,10 +992,11 @@ void * ret;
 		return 0;
 
 	change_endian(_r->type);
+	change_endian(_r->size);
 	switch ( _r->type ) {
 	case PNT_MX_DATA_1:
 		d1 = (PN64_MX_DATA_1*)_r;
-		ret = (*tp->convert_to_host)(tp,d1+1,MD_DALLOC,0);
+		ret = (*tp->convert_to_host)(tp,d1+1,d1->h.size-sizeof(*d1),MD_DALLOC,0,__FILE__,__LINE__);
 		break;
 	case PNT_MX_DATA_2:
 		d2 = (PN64_MX_DATA_2*)_r;
@@ -986,7 +1009,7 @@ void * ret;
 			(m->channel_info[d2->id].data_type->type);
 		if ( tp == 0 )
 			goto err;
-		ret = (*tp->convert_to_host)(tp,d2+1,MD_DALLOC,0);
+		ret = (*tp->convert_to_host)(tp,d2+1,d2->h.size-sizeof(*d2),MD_DALLOC,0,__FILE__,__LINE__);
 		if ( next_fofs )
 			*next_fofs = d2->next_fofs;
 		if ( id )
@@ -1069,6 +1092,7 @@ unsigned char * ptr;
 	return 0;
 }
 
+/*
 int
 save_raw_matrix_node(
 	MATRIX * m,
@@ -1107,7 +1131,7 @@ MX_FILE_WORK * w;
 	finish_file_access(m,0);
 	return 0;
 }
-
+*/
 
 int
 delete_mx_cache_ring(PN64_MX_CACHE_RING * dr,MATRIX *m,INTEGER64 cr_fofs)
@@ -1378,6 +1402,8 @@ INTEGER64 fofs,next_fofs;
 int id;
 void * d;
 FAVT64_ROOT * rt;
+INTEGER64 * dim_code;
+int i;
 
 	m = n->matrix;
 
@@ -1407,15 +1433,26 @@ FAVT64_ROOT * rt;
 	n->fofs = dc->fofs;
 	wakeup_task((int)n);
 
+	dim_code = d_alloc(sizeof(INTEGER64)*(m->p.dim+1));
+
 	if ( m->dim_bit_field ) {
 		load_raw_matrix_node(
-			&mx_n,&org,0,n->nlist_dim_bit_field,m,dc->fofs);
+			&mx_n,&org,dim_code,n->nlist_dim_bit_field,m,dc->fofs);
 	}
 	else {
-		load_raw_matrix_node(&mx_n,&org,0,0,m,dc->fofs);
+		load_raw_matrix_node(&mx_n,&org,dim_code,0,m,dc->fofs);
 		load_matrix_dim_addr(n,mx_n.d.p3.f.nlist_dim_addr);
 		n->dim_addr_fofs = mx_n.d.p3.f.nlist_dim_addr;
 	}
+	if ( dim_code[0] != n->dim_code[0] )
+		er_panic("loading_error(1)");
+	for ( i = 0; i < m->p.dim ; i ++ ) {
+	INTEGER64 mask;
+		mask = -(((INTEGER64)1)<<((m->dim_divide[i]*dim_code[0])+m->block_size[i]));
+		if ( (dim_code[i+1] & mask) != (n->dim_code[i+1] & mask) )
+			er_panic("loading_error(2)");
+	}
+	d_f_ree(dim_code);
 
 	if ( m->p.flags & MPF_CACHE_FILE ) {
 		delete_mx_cache_ring(0,m,mx_n.d.p3.c.ring_fofs);
@@ -1481,13 +1518,15 @@ INTEGER64 _ring,_dim_addr;
 MX_DIM_CODE * dc;
 FAVT64_NODE * fn, * fn2;
 FAVT64_ROOT * rt;
+int er;
 
 PN64_HEADER * h;
 
+int debug;
 
 	m = n->matrix;
-	if ( start_file_access(m) < 0 )
-		return -1;
+	if ( (er=start_file_access(m)) < 0 )
+		return er;
 
 	w = m->file_work;
 
@@ -1575,8 +1614,9 @@ PN64_HEADER * h;
 			m->dim_bit_field,0);
 
 	setup_recordlist64(rl);
-
+debug = 0;
 	if ( n->fofs == 0 ) {
+debug = 1;
 		n->fofs = alloc_filespace64(w->pdb_fd,rl->data);
 		if ( n->ring_fofs )
 			setup_mx_cache_ring(m,n->ring_fofs,n->fofs);
@@ -1609,6 +1649,8 @@ PN64_HEADER * h;
 		
 	}
 	h = rl->data;
+
+//ss_printf("NODE SIZE " I64_FORMAT "\n",h->size);
 	change_endian_header64(h);
 	write_filespace64f(w->pdb_fd,n->fofs,rl->data);
 	wakeup_task((int)n);
@@ -1647,9 +1689,10 @@ close_matrix_file(MATRIX * m)
 int
 matrix_standard_write_file(MATRIX_TOKEN * t)
 {
-	save_matrix_node(t->process_node);
+int ret;
+	ret = save_matrix_node(t->process_node);
 	unlock_save(t->process_node);
-	return 0;
+	return ret;
 }
 
 int 
@@ -1673,7 +1716,7 @@ retry:
 	flush_all_dirty_file(m);
 	for ( i = 0 ; i < NHASH_SIZE ; i ++ ) {
 		for ( n = m->node_hash[i] ; n ; n = n->nh_next )
-			if ( n->fofs == 0 && n->status <= MS_OK ) {
+			if ( n->fofs == 0 && n->status < MS_OK ) {
 				finish_file_access_with_wait(m,(int)n);
 				if ( start_file_access(m) < 0 )
 					return -1;
@@ -1788,34 +1831,40 @@ INTEGER64 fofs;
 STREAM * st;
 
 	if ( mode < 0 || mode >= MI_MODE_MAX ) {
-		memset(pri,0,sizeof(int)*MI_MAX);
-		memset(cals,0,sizeof(XL_SEXP*)*MI_MAX);
+		if ( pri )
+			memset(pri,0,sizeof(int)*MI_MAX);
+		if ( cals )
+			memset(cals,0,sizeof(XL_SEXP*)*MI_MAX);
 		return -1;
 	}
 
 	if ( start_file_access(m) < 0 )
-		return -1;
+		return -2;
 	w = m->file_work;
 	
-	memcpy(pri,w->subheader.mode_pri_area[mode],sizeof(int)*MI_MAX);
+	if ( pri )
+		memcpy(pri,w->subheader.mode_pri_area[mode],sizeof(int)*MI_MAX);
 	
-	for ( i = 0 ; i < MI_MAX ; i ++ ) {
-		fofs = w->subheader.mode_cal_fofs[mode][i];
-		if ( fofs == 0 ) {
-			cals[i] = 0;
-			continue;
+	if ( cals ) {
+		for ( i = 0 ; i < MI_MAX ; i ++ ) {
+			fofs = w->subheader.mode_cal_fofs[mode][i];
+			if ( fofs == 0 ) {
+				cals[i] = 0;
+				continue;
+			}
+			c = read_filespace64f(w->pdb_fd,fofs);
+			if ( c == 0 ) {
+				cals[i] = 0;
+				continue;
+			}
+			change_endian_header64(&c->h);
+			st = s_open_string_read(c+1,&utf8_cm,c->h.size - sizeof(PN64_HEADER),1);
+			ret = init_parse(st,
+				l_string(std_cm,"sexp_commands"),
+				l_string(std_cm,"sexp_commands"));
+			for ( s = ret ; get_type(s) == XLT_PAIR ; s = cdr(s) );
+			cals[i] = ret;
 		}
-		c = read_filespace64f(w->pdb_fd,fofs);
-		if ( c == 0 ) {
-			cals[i] = 0;
-			continue;
-		}
-		st = s_open_string_read(c+1,&utf8_cm,c->h.size - sizeof(PN64_HEADER),1);
-		ret = init_parse(st,
-			l_string(std_cm,"sexp_commands"),
-			l_string(std_cm,"sexp_commands"));
-		for ( s = ret ; get_type(s) == XLT_PAIR ; s = cdr(s) );
-		cals[i] = ret;
 	}
 	finish_file_access(m,0);
 	return 0;
@@ -1836,52 +1885,58 @@ int f;
 		return -1;
 
 	if ( start_file_access(m) < 0)
-		return -1;
+		return -2;
 	w = m->file_work;
 	
-	memcpy(w->subheader.mode_pri_area[mode],pri,sizeof(int)*MI_MAX);
-	
 	f = 0;
-	for ( i = 0 ; i < MI_MAX ; i ++ ) {
-		if ( cals[i] ) {
-			rl = new_recordlist64(PNT_MX_CAL,sizeof(PN64_MX_CAL));
-			h = rl->data;
+	if ( pri ) {
+		f = 1;
+		memcpy(w->subheader.mode_pri_area[mode],pri,sizeof(int)*MI_MAX);
+	}
+	if ( cals ) {
+		for ( i = 0 ; i < MI_MAX ; i ++ ) {
+			if ( cals[i] ) {
+				rl = new_recordlist64(PNT_MX_CAL,sizeof(PN64_MX_CAL));
+				h = rl->data;
 
-			for ( j = 0 ; cals[i][j] ; j ++ )
-				set_recordlist_chain64(rl,
-					cals[i][j],strlen(cals[i][j]),0);
-			setup_recordlist64(rl);
-			h = rl->data;
-			
-			fofs = w->subheader.mode_cal_fofs[mode][i];
-			if ( fofs == 0 ) {
-			new_alloc:
-				f = 1;
-				fofs = alloc_filespace64(w->pdb_fd,rl->data);
+				for ( j = 0 ; cals[i][j] ; j ++ )
+					set_recordlist_chain64(rl,
+						cals[i][j],strlen(cals[i][j]),0);
+				setup_recordlist64(rl);
+				h = rl->data;
+				
+				fofs = w->subheader.mode_cal_fofs[mode][i];
+				if ( fofs == 0 ) {
+				new_alloc:
+					f = 1;
+					fofs = alloc_filespace64(w->pdb_fd,rl->data);
+				}
+				else {
+					c = read_filespace64f(w->pdb_fd,fofs);
+					if ( c == 0 ) {
+						goto new_alloc;
+					}
+					change_endian_header64(&c->h);
+					if ( h->h.size != c->h.size ) {
+						d_f_ree(c);
+						free_filespace64(w->pdb_fd,fofs);
+						goto new_alloc;
+					}
+					d_f_ree(c);
+				}
+				change_endian_header64(rl->data);
+				write_filespace64f(w->pdb_fd,fofs,rl->data);
+				w->subheader.mode_cal_fofs[mode][i] = fofs;
+				
+				free_recordlist64(rl);
 			}
 			else {
-				c = read_filespace64f(w->pdb_fd,fofs);
-				if ( c == 0 ) {
-					goto new_alloc;
-				}
-				change_endian_header64(&c->h);
-				if ( h->h.size != c->h.size ) {
-					d_f_ree(c);
+				fofs = w->subheader.mode_cal_fofs[mode][i];
+				if ( fofs ) {
 					free_filespace64(w->pdb_fd,fofs);
-					goto new_alloc;
+					w->subheader.mode_cal_fofs[mode][i] = 0;
+					f = 1;
 				}
-				d_f_ree(c);
-			}
-			change_endian_header64(rl->data);
-			write_filespace64f(w->pdb_fd,fofs,rl->data);
-			w->subheader.mode_cal_fofs[mode][i] = fofs;
-			
-			free_recordlist64(rl);
-		}
-		else {
-			fofs = w->subheader.mode_cal_fofs[mode][i];
-			if ( fofs ) {
-				free_filespace64(w->pdb_fd,fofs);
 			}
 		}
 	}
@@ -1891,5 +1946,12 @@ int f;
 	return 0;
 }
 
+PDB64 *
+get_mx_file_pfd(MATRIX * m)
+{
+MX_FILE_WORK * w;
+	w = m->file_work;
+	return w->pdb_fd;
+}
 
 
