@@ -32,6 +32,8 @@ xx_mxt_alloc_copy(MATRIX_DATA_TYPE * tp,void * d,int atype,void * at_work,char *
 int size;
 void * ret;
 
+	if ( d == 0 )
+		return 0;
 	size = (*tp->get_size)(tp,d);
 	if ( atype == MD_MMALLOC )
 		ret = xx_atype_alloc(size,atype,gc_text,__f,__l);
@@ -517,14 +519,14 @@ int t;
 }
 
 void *
-mxt_convert_basic_to_host(MATRIX_DATA_TYPE * tp,void * d,int atype,void * at_work)
+mxt_convert_basic_to_host(MATRIX_DATA_TYPE * tp,void * d,int _size,int atype,void * at_work,char * __f,int __l)
 {
 void * ret;
 int size;
 
 	if ( tp->type & MDT_VECTOR )
-		ret = atype_alloc(size=get_size_endian(d),atype,at_work);
-	else	ret = atype_alloc(size=(*tp->get_size)(tp,d),atype,at_work);
+		ret = xx_atype_alloc(size=get_size_endian(d),atype,at_work,__f,__l);
+	else	ret = xx_atype_alloc(size=(*tp->get_size)(tp,d),atype,at_work,__f,__l);
 	memcpy(ret,d,size);
 	(*tp->endian_to_host)(ret);
 	return ret;
@@ -897,3 +899,24 @@ unsigned int _tim;
 		ss_printf(" :: %s|VECTOR\n",tpp);
 	else	ss_printf(" :: %s|SCOLOR\n",tpp);
  }
+ 
+ 
+ 
+
+void
+set_recordlist_code(RECORD_LIST64 * rl,CHAIN_LIST64 * ptr)
+{
+int size;
+CHAIN_LIST64 * p;
+unsigned char * buffer;
+unsigned char * pp,*qq;
+	size = 0;
+	for ( p = ptr->next ; p ; p = p->next )
+		size += p->len;
+ss_printf("SRC %i\n",size);
+	pp = buffer = d_alloc(sizeof(INTEGER64)*2+1);
+	qq = get_compressed_code64(pp,size);
+	set_recordlist_chain64_middle(rl,ptr,buffer,qq - pp,1);
+}
+
+
