@@ -1,4 +1,4 @@
-/* ver.B.b11.02(BLAZE) */
+/* ver.B.b14(BLAZE) */
 
 
 #include "version.h"
@@ -7,9 +7,9 @@
 VERSION ap_version = {
 	"AP.code",
 	"r64status",
-	"(C) 2000-2005 Hirohisa Mori",
-	"ver.B.b11.02(BLAZE)",
-	2006,8,21,
+	"(C) 2000-2007 Hirohisa Mori",
+	"ver.B.b14(BLAZE)",
+	2007,2,10,
 	0,
 	"r64/cr format status viewer ",
 };
