@@ -1,4 +1,4 @@
-/* ver.B.b11.02(BLAZE) */
+/* ver.B.b14(BLAZE) */
 
 
 #include "version.h"
@@ -7,9 +7,9 @@
 VERSION machine_code_version = {
 	"MA.code",
 	"posix.linux.Interface",
-	"(C) 2000-2005 Hirohisa Mori",
-	"ver.B.b11.02(BLAZE)",
-	2006,8,21,
+	"(C) 2000-2007 Hirohisa Mori",
+	"ver.B.b14(BLAZE)",
+	2007,2,13,
 	0,
 	"machine (posix.linux) dependent library ",
 };
